/*
 *	dq12spell.c
 *		Copyright (C)2001-2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "dq12decode.h"
#include "dq12spell.h"

#if defined(MODE_DQ1)
#define FILEPOS_SPELLNAME_ADDR		0x05bd8e
#define NUM_SPELL					10
#else /* MODE_DQ1 */
#define FILEPOS_SPELLNAME_ADDR		0x05bda2
#define NUM_SPELL					22
#endif /* MODE_DQ1 */

#define FILEPOS_SPELLNAME_BANK		0x000000
#define FILEPOS_SPELLACTNAME_ADDR	0x058ba0
#define NUM_SPELLACT				17
#define SIZEOF_OFFSETADDR			2

static void spell_setname(void);

static char tbl_spellname[NUM_SPELL][256];			/* ̖Oi[e[u */
static char tbl_spellactname[NUM_SPELLACT][256];	/* sp^[p̎i[e[u */
static int flag_setspellname = 0;					/* tbl_spellnameEtbl_spellactnameɃf[^i[Ă邩ǂ */

const char *spell_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_SPELL <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setspellname)
		spell_setname();

	return tbl_spellname[index];
}

static void spell_setname(void)
{
	int i;
	int tbl_offset_n[NUM_SPELL];
	int tbl_offset_a[NUM_SPELLACT];

	/* ̖Of[^ */
	for (i = 0; i < NUM_SPELL; i++)
		*tbl_spellname[i] = '\0';
	for (i = 0; i < NUM_SPELLACT; i++)
		*tbl_spellactname[i] = '\0';

	/* ̖Of[^i[ĂAhX(ւ̃ItZbg)ǂݍ */
	for (i = 0; i < NUM_SPELL; i++)
	{
		unsigned char buf[SIZEOF_OFFSETADDR];

		file_read_sfc(buf, FILEPOS_SPELLNAME_ADDR + SIZEOF_OFFSETADDR * i, SIZEOF_OFFSETADDR);
		tbl_offset_n[i] = (buf[1] << 8) | buf[0];
	}

	/* sp^[p̎f[^i[ĂAhX(ւ̃ItZbg)ǂݍ */
	for (i = 0; i < NUM_SPELLACT; i++)
	{
		unsigned char buf[SIZEOF_OFFSETADDR];

		file_read_sfc(buf, FILEPOS_SPELLACTNAME_ADDR + SIZEOF_OFFSETADDR * i, SIZEOF_OFFSETADDR);
		tbl_offset_a[i] = (buf[1] << 8) | buf[0];
	}

	/* ̖Of[^ǂݍ */
	for (i = 0; i < NUM_SPELL; i++)
	{
		unsigned char buf[256];
		unsigned char *ptr_term = buf;

		file_read_sfc(buf, FILEPOS_SPELLNAME_BANK + tbl_offset_n[i], sizeof(buf));

		if ((ptr_term = memchr(buf, 0xff, sizeof(buf))) == NULL)
			break;

		dec_string(tbl_spellname[i], buf, sizeof(tbl_spellname[i]), ptr_term - buf, 0);
	}

	/* sp^[p̎f[^ǂݍ */
	for (i = 0; i < NUM_SPELLACT; i++)
	{
		unsigned char buf[256];
		unsigned char *ptr_term = buf;

		file_read_sfc(buf, FILEPOS_SPELLNAME_BANK + tbl_offset_a[i], sizeof(buf));

		if ((ptr_term = memchr(buf, 0xff, sizeof(buf))) == NULL)
			break;

		dec_string(tbl_spellactname[i], buf, sizeof(tbl_spellactname[i]), ptr_term - buf, 0);
	}

	flag_setspellname = 1;
}

const char *spell_getactname(int index)
{
	/* `FbN */
	if (index < 0 || 32 <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setspellname)
		spell_setname();

	return (6 <= index && index < 23) ? tbl_spellactname[index - 6] : "";
}

const char *spell_getactdesc(int index)
{
	static const char *tbl_strdesc[] =
	{
		/* 00 */	"U",
		/* 01 */	"ɍ̈ꌂ",
		/* 02 */	"ōU",
		/* 03 */	"U",
		/* 04 */	"h",
		/* 05 */	"",
		/* 06 */	"",
		/* 07 */	"",
		/* 08 */	"",
		/* 09 */	"g",
		/* 0a */	"g",
		/* 0b */	"g",
		/* 0c */	"ȊO",
		/* 0d */	"ȊO",
		/* 0e */	"ȊO",
		/* 0f */	"",
		/* 10 */	"",
		/* 11 */	"",
		/* 12 */	"",
		/* 13 */	"",
		/* 14 */	"",
		/* 15 */	"",
		/* 16 */	"",
		/* 17 */	"Ђ̂",
		/* 18 */	"邩",
		/* 19 */	"͂ق̂",
		/* 1a */	"ǂ̂",
		/* 1b */	"܂",
		/* 1c */	"ԂĂ",
		/* 1d */	"2U",
		/* 1e */	"WU",
		/* 1f */	"ӂȂǂ",
	};

	assert(sizeof(tbl_strdesc) / sizeof(tbl_strdesc[0]) == 32);

	/* `FbN */
	if (index < 0 || 32 <= index)
	{
		assert(0);
		return "";
	}

	return tbl_strdesc[index];
}
