/*
 *	dq5item.c
 *		Copyright (C)2001-2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include "errmes.h"
#include "filesfc.h"
#include "dq5decode.h"
#include "dq5item.h"

#define NUM_ITEM			216
#define FILEPOS_ITEMNAME	0x11ce0e
#define FILEPOS_ITEMVALUE	0x11b4b9
#define FILEPOS_ITEMPRICE1	0x11b081
#define FILEPOS_ITEMPRICE2	0x11b159
#define FILEPOS_ITEMDEC		0x1380fc
#define SIZEOF_ITEMVALUE	1
#define SIZEOF_ITEMPRICE1	1
#define SIZEOF_ITEMPRICE2	1
#define SIZEOF_ITEMDEC		3

static void item_setname(void);

static char tbl_itemname[NUM_ITEM][256];	/* ̖Oi[e[u */
static int flag_setitemname = 0;			/* tbl_itemnameɃf[^i[Ă邩ǂ */

const char *item_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_ITEM <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setitemname)
		item_setname();

	return tbl_itemname[index];
}

static void item_setname(void)
{
	int i;
	long filepos;

	/* ̖Of[^ǂݍ */
	filepos = FILEPOS_ITEMNAME;
	for (i = 0; i < NUM_ITEM; i++)
	{
		unsigned char byte;
		unsigned char buf[256];
		size_t readsize = 0;

		filepos += file_read_sfc(&byte, filepos, 1);
		readsize = (size_t)byte;

		if (readsize > 0)
		{
			filepos += file_read_sfc(buf, filepos, readsize);
			dec_string(tbl_itemname[i], buf, sizeof(tbl_itemname[i]), readsize, 0);
		}
	}

	flag_setitemname = 1;
}

itemdata_t *item_readdata(void)
{
	int i, j;
	long filepos;
	itemdata_t *buf_itemdata = NULL;

	/* ̊ef[^i[obt@mۂ */
	if ((buf_itemdata = malloc(sizeof(itemdata_t) * NUM_ITEM)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* ̊ef[^ǂݍ */
	for (i = 0; i < NUM_ITEM; i++)
	{
		/* ㏸l */
		{
			unsigned char byte;

			file_read_sfc(&byte, FILEPOS_ITEMVALUE + SIZEOF_ITEMVALUE * i, SIZEOF_ITEMVALUE);

			buf_itemdata[i].increase_value = byte;
		}

		/* i */
		{
			unsigned char byte1;
			unsigned char byte2;

			file_read_sfc(&byte1, FILEPOS_ITEMPRICE1 + SIZEOF_ITEMPRICE1 * i, SIZEOF_ITEMPRICE1);
			file_read_sfc(&byte2, FILEPOS_ITEMPRICE2 + SIZEOF_ITEMPRICE2 * i, SIZEOF_ITEMPRICE2);

			if (byte1 == 0x00)
				buf_itemdata[i].price = byte2 & 0x03;
			else
			{
				long price = byte1;

				for (j = 0; j < (byte2 & 0x03); j++)
					price *= 10;

				buf_itemdata[i].price = price;
			}
		}

		/* _[Wyf[^ */
		buf_itemdata[i].decrease_point = 0;
		buf_itemdata[i].elem_decrease  = 0UL;
	}

	/* (h)̃_[Wyf[^ǂݍ */
	filepos = FILEPOS_ITEMDEC;
	for (;;)
	{
		unsigned char buf[SIZEOF_ITEMDEC];

		filepos += file_read_sfc(buf, filepos, SIZEOF_ITEMDEC);
		if (buf[0] == 0x00)
			break;

		buf_itemdata[(int)buf[1]].decrease_point = ((buf[2] & 0x80) == 0) ? (int)buf[2] : (int)buf[2] - 0x100;
		buf_itemdata[(int)buf[1]].elem_decrease  = (unsigned long)buf[0];
	}

	return buf_itemdata;
}

int item_writeparam(const char *ptr_filename, const itemdata_t *ptr_itemdata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_itemdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* ̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* ̐lnf[^o͂ */
	fprintf(fp, "No.      O                 "
				"  i ㏸l\n");
	for (i = 0; i < NUM_ITEM; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s : %6ld, %5d\n", i, i,
				item_getname(i),
				ptr_itemdata[i].price,
				ptr_itemdata[i].increase_value);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
