/*
 *	dq5monster.c
 *		Copyright (C)2001-2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "dq5decode.h"
#include "dq5item.h"
#include "dq5spell.h"
#include "dq5monster.h"

#define FILEPOS_MONNAME		0x11c69c
#define FILEPOS_MONDATA		0x118000
#define FILEPOS_MONENC		0x1196f3
#define FILEPOS_MONMAP		0x11f1fc
#define NUM_MON				235
#define NUM_MONENC			85
#define NUM_MONMAP			5
#define SIZEOF_MONDATA		25
#define SIZEOF_MONENC		22

static void mon_setname(void);

static char tbl_monname[NUM_MON][256];	/* X^[̖Oi[e[u */
static int flag_setmonname = 0;			/* tbl_monnameɃf[^i[Ă邩ǂ */

static const struct
{
	int width;
	int height;
} tbl_sizeof_monmap[NUM_MONMAP] =
{
	{17,  7},
	{17, 17},
	{17, 17},
	{17,  5},
	{17,  4},
};

const char *mon_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_MON <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setmonname)
		mon_setname();

	return tbl_monname[index];
}

static void mon_setname(void)
{
	int i;
	long filepos;

	/* X^[̖Of[^ */
	for (i = 0; i < NUM_MON; i++)
		*tbl_monname[i] = '\0';

	/* X^[̖Of[^ǂݍ */
	filepos = FILEPOS_MONNAME;
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char byte;
		unsigned char buf[256];
		size_t readsize = 0;

		filepos += file_read_sfc(&byte, filepos, 1);
		readsize = (size_t)byte;

		if (readsize > 0)
		{
			filepos += file_read_sfc(buf, filepos, readsize);
			dec_string(tbl_monname[i], buf, sizeof(tbl_monname[i]), readsize, 0);
		}
	}

	flag_setmonname = 1;
}

mondata_t *mon_readdata(void)
{
	int i, j;
	mondata_t *buf_mondata = NULL;

	/* X^[̊ef[^i[obt@mۂ */
	if ((buf_mondata = malloc(sizeof(mondata_t) * NUM_MON)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[̊ef[^ǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONDATA];

		file_read_sfc(buf, FILEPOS_MONDATA + SIZEOF_MONDATA * i, SIZEOF_MONDATA);

		/* lnf[^ */
		buf_mondata[i].hit_point = ((buf[10] & 0x07) << 8) | buf[ 3];

		/* ʈ */
		if (buf_mondata[i].hit_point == 2046)	/* $21A307 */
			buf_mondata[i].hit_point = 4500;	/* $21A30C */
		if (buf_mondata[i].hit_point == 2045)	/* $21A311 */
			buf_mondata[i].hit_point = 9000;	/* $21A316 */
		if (buf_mondata[i].hit_point == 2044)	/* $21A31B */
			buf_mondata[i].hit_point = 3000;	/* $21A320 */

		buf_mondata[i].magic_power      = buf[ 4];
		buf_mondata[i].experience_point = (buf[ 6] << 8) | buf[ 5];
		buf_mondata[i].gold             = ((buf[ 8] & 0x0c) << 6) | buf[ 7];
		buf_mondata[i].attack_power     = ((buf[ 8] & 0x80) << 1) | buf[ 1];
		buf_mondata[i].defense_power    = ((buf[ 8] & 0x40) << 2) | buf[ 2];
		buf_mondata[i].agility          = buf[ 0];

		/* ϐnf[^ */
		buf_mondata[i].regist_blaze     = (buf[12] & 0x30) >> 4;
		buf_mondata[i].regist_firebane  = (buf[12] & 0x0c) >> 2;
		buf_mondata[i].regist_explodet  = buf[12] & 0x03;
		buf_mondata[i].regist_snowstorm = (buf[13] & 0xc0) >> 6;
		buf_mondata[i].regist_infernos  = (buf[13] & 0x30) >> 4;
		buf_mondata[i].regist_lightning = (buf[13] & 0x0c) >> 2;
		buf_mondata[i].regist_defeat    = buf[13] & 0x03;
		buf_mondata[i].regist_sleep     = (buf[14] & 0xc0) >> 6;
		buf_mondata[i].regist_surround  = (buf[14] & 0x30) >> 4;
		buf_mondata[i].regist_expell    = (buf[14] & 0x0c) >> 2;
		buf_mondata[i].regist_defense   = buf[14] & 0x03;
		buf_mondata[i].regist_robmagic  = (buf[15] & 0xc0) >> 6;
		buf_mondata[i].regist_stopspell = (buf[15] & 0x30) >> 4;
		buf_mondata[i].regist_chaos     = (buf[15] & 0x0c) >> 2;
		buf_mondata[i].regist_sacrifice = buf[15] & 0x03;
		buf_mondata[i].regist_unknown   = (buf[16] & 0xc0) >> 6;
		buf_mondata[i].regist_poison    = (buf[16] & 0x30) >> 4;
		buf_mondata[i].regist_stun      = (buf[16] & 0x0c) >> 2;

		/* ̑̃f[^ */
		buf_mondata[i].chr_no            = buf[24];
		buf_mondata[i].chr_join          = buf[ 9] & 0x07;
		buf_mondata[i].item_no           = buf[23];
		buf_mondata[i].item_get          = (buf[ 9] & 0x38) >> 3;
		buf_mondata[i].auto_healing      = (buf[10] & 0x18) >> 3;
		buf_mondata[i].repeatedly_action = (buf[11] & 0xc0) >> 6;
		buf_mondata[i].apriori_statecng  = ((buf[12] & 0x80) == 0x00) ? 0 : 1;
		buf_mondata[i].rate_statecng     = (buf[ 9] & 0xc0) >> 6;
		buf_mondata[i].action_type       = (buf[10] & 0xe0) >> 5;
		buf_mondata[i].judgment      = buf[ 8] & 0x03;

		/* sp^[f[^ */
		for (j = 0; j < 6; j++)
			buf_mondata[i].action[j] = buf[j + 17];
	}

	return buf_mondata;
}

monenc_t *mon_readenc(void)
{
	int i, j;
	monenc_t *buf_monenc = NULL;

	/* X^[e[uf[^i[obt@mۂ */
	if ((buf_monenc = malloc(sizeof(monenc_t) * NUM_MONENC)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[e[uf[^ǂݍ */
	for (i = 0; i < NUM_MONENC; i++)
	{
		unsigned char buf[SIZEOF_MONENC];

		file_read_sfc(buf, FILEPOS_MONENC + SIZEOF_MONENC * i, SIZEOF_MONENC);

		buf_monenc[i].level = buf[0];

		for (j = 0; j < 10; j++)
		{
			buf_monenc[i].mon[j].monster_no = buf[j + 12];

			buf_monenc[i].mon[j].unknown1 = buf[j + 1] >> 6;
			buf_monenc[i].mon[j].unknown2 = (buf[j + 1] & 0x38) >> 3;
			buf_monenc[i].mon[j].unknown3 = buf[j + 1] & 0x07;
		}
	}

	return buf_monenc;
}

monmap_t *mon_readmap(void)
{
	int i, x, y;
	unsigned char *buf = NULL;
	unsigned char *ptr_buf = NULL;
	monmap_t *buf_monmap = NULL;
	const size_t bufsize = 1024;

	/* X^[}bvf[^i[obt@mۂ */
	if ((buf_monmap = malloc(sizeof(monmap_t) * NUM_MONMAP)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);
	for (i = 0; i < NUM_MONMAP; i++)
	{
		for (y = 0; y < 17; y++)
		{
			for (x = 0; x < 17; x++)
				buf_monmap[i].area[x][y] = 0;
		}
	}

	/* X^[}bvf[^ǂݍ */
	if ((buf = malloc(bufsize)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);
	memset(buf, 0x00, bufsize);
	file_read_sfc(buf, FILEPOS_MONMAP, bufsize);

	ptr_buf = buf;
	for (i = 0; i < NUM_MONMAP; i++)
	{
		for (y = 0; y < tbl_sizeof_monmap[i].height; y++)
		{
			for (x = 0; x < tbl_sizeof_monmap[i].width; x++)
				buf_monmap[i].area[x][y] = *ptr_buf++;
		}
	}

	free(buf);
	buf = NULL;

	return buf_monmap;
}

int mon_writeparam(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̐lnf[^o͂ */
	fprintf(fp, "No.      O                 "
				"    HP     MP     Ex      G "
				"U  f\n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s :  %5d, %5d, %5d, %5d, %5d, %5d, %5d\n", i, i,
				mon_getname(i),
				ptr_mondata[i].hit_point,
				ptr_mondata[i].magic_power,
				ptr_mondata[i].experience_point,
				ptr_mondata[i].gold,
				ptr_mondata[i].attack_power,
				ptr_mondata[i].defense_power,
				ptr_mondata[i].agility);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeregist(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̑ϐnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̑ϐnf[^o͂ */
	fprintf(fp, "No.      O                 "
				"Ή M  C ^ d V Ö f ގU 猸 MPz    s    \n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s :  %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\n", i, i,
				mon_getname(i),
				ptr_mondata[i].regist_blaze,
				ptr_mondata[i].regist_firebane,
				ptr_mondata[i].regist_explodet,
				ptr_mondata[i].regist_snowstorm,
				ptr_mondata[i].regist_infernos,
				ptr_mondata[i].regist_lightning,
				ptr_mondata[i].regist_defeat,
				ptr_mondata[i].regist_sleep,
				ptr_mondata[i].regist_surround,
				ptr_mondata[i].regist_expell,
				ptr_mondata[i].regist_defense,
				ptr_mondata[i].regist_robmagic,
				ptr_mondata[i].regist_stopspell,
				ptr_mondata[i].regist_chaos,
				ptr_mondata[i].regist_sacrifice,
				ptr_mondata[i].regist_unknown,
				ptr_mondata[i].regist_poison,
				ptr_mondata[i].regist_stun);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeitem(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;
	const char *str_nothing = "------------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̕󔠂̓f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̕󔠂̓f[^o͂ */
	fprintf(fp, "No.      O                 "
				"x   \n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s :  %3d,  %-18s\n", i, i,
				mon_getname(i),
				ptr_mondata[i].item_get,
				(ptr_mondata[i].item_no != 0xff) ? item_getname(ptr_mondata[i].item_no) : str_nothing);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeact(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "";
	const char *str_repeat[] =
	{
		"1",
		"1`2",
		"2",
		"3",
	};
	const char *str_pattern[] =
	{
		"_",
		"s2̊mAȉ1, 3, 4, 5, 6̏",
		"s1̊m|IɍAȉ2, 3, 4, 5, 6̏",
		"s1, 2, 3̊m",
		"_",
		"[e[V(s1  2  3  4  5  6)",
		"[e[V(s1܂2  3܂4  5܂6)",
		"[e[V(s1  2  3  4  5  6)",
	};

	assert(sizeof(str_repeat) / sizeof(str_repeat[0]) == 4);
	assert(sizeof(str_pattern) / sizeof(str_pattern[0]) == 8);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̍sp^[f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̍sp^[f[^o͂ */
	for (i = 0; i < NUM_MON; i++)
	{
		if (ptr_mondata[i].repeatedly_action >= 4 ||
			ptr_mondata[i].action_type       >= 8)
		{
			break;
		}

		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%03d(%02X): %s]\n", i, i, mon_getname(i));
		fprintf(fp, "s:   %s\n", str_repeat[ptr_mondata[i].repeatedly_action]);
		fprintf(fp, "s^Cv: %d (%s)\n", ptr_mondata[i].action_type, str_pattern[ptr_mondata[i].action_type]);

		fprintf(fp, "No. s\n");
		for (j = 0; j < 6; j++)
		{
			const char *ptr_spellname = (ptr_mondata[i].action[j] != 0xff) ? spell_getname(ptr_mondata[i].action[j]) : str_nothing;
			const char *ptr_spelldesc = (ptr_mondata[i].action[j] != 0xff) ? spell_getdesc(ptr_mondata[i].action[j]) : str_nothing;

			fprintf(fp, "%02d:", j + 1);
			if (strlen(ptr_spellname) > 0)
				fprintf(fp, " %s", ptr_spellname);
			if (strlen(ptr_spelldesc) > 0)
				fprintf(fp, " (%s)", ptr_spelldesc);
			fputc('\n', fp);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writemisc(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;
	const char *str_state[] = {"", "}zJ^"};

	assert(sizeof(str_state) / sizeof(str_state[0]) == 2);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̂̑̃f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̂̑̃f[^o͂ */
	fprintf(fp, "No.      O                 "
				"  f ԕω\n");
	for (i = 0; i < NUM_MON; i++)
	{
		if (ptr_mondata[i].apriori_statecng >= 2)
			break;

		fprintf(fp, "%03d(%02X): %-18s :  %3d, %3d, %3d, %3d", i, i,
				mon_getname(i),
				ptr_mondata[i].chr_join,
				ptr_mondata[i].auto_healing,
				ptr_mondata[i].judgment,
				ptr_mondata[i].rate_statecng);
		if (ptr_mondata[i].rate_statecng > 0)
			fprintf(fp, ", %s", str_state[ptr_mondata[i].apriori_statecng]);
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeenc(const char *ptr_filename, const monenc_t *ptr_monenc)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "------------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_monenc == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[e[uf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[e[uo͂ */
	for (i = 0; i < NUM_MONENC; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[%03d(%02X)]\n", i, i);
		fprintf(fp, "x: %3d\n", ptr_monenc[i].level);
		fprintf(fp, "No. X^[\n");
		for (j = 0; j < 10; j++)
		{
			fprintf(fp, "%02d: %-18s\n", j,
					(ptr_monenc[i].mon[j].unknown3 != 0) ? mon_getname(ptr_monenc[i].mon[j].monster_no) : str_nothing);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writemap(const char *ptr_filename, const monmap_t *ptr_monmap)
{
	int i, x, y;
	FILE *fp = NULL;
	const char *str_name[/* NUM_MONMAP */] =
	{
		"cN",
		"NO",
		"N㔼",
		"d̑",
		"E",
	};

	assert(sizeof(str_name) / sizeof(str_name[0]) == NUM_MONMAP);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_monmap == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[}bvf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[}bvo͂ */
	for (i = 0; i < NUM_MONMAP; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%1d: %s]\n", i, str_name[i]);
		for (y = 0; y < tbl_sizeof_monmap[i].height; y++)
		{
			for (x = 0; x < tbl_sizeof_monmap[i].width; x++)
			{
				if (x != 0)
					fprintf(fp, " ");

				if (ptr_monmap[i].area[x][y] != 0x00)
					fprintf(fp, "%03d", ptr_monmap[i].area[x][y]);
				else
					fprintf(fp, "---");
			}
			fputc('\n', fp);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
