/*
 *	dq6character.c
 *		Copyright (C)2001-2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "dq6define.h"
#include "dq6item.h"
#include "dq6character.h"

#define FILEPOS_CHRDATA		0x08bd12
#define FILEPOS_CHRLVUP		0x3fb6c2
#define NUM_CHR				82
#define NUM_CHRTYPE			27
#define SIZEOF_CHRDATA		27
#define SIZEOF_CHRLVUP		177

static void chr_setnameindex(void);

static int tbl_chrnameindex[NUM_CHR];	/* LN^\̃CfbNXi[e[u */
static int flag_chrnameindex = 0;		/* tbl_chrnameindexɃf[^i[Ă邩ǂ */

static const int tbl_no_to_type[NUM_CHR] =
{
	 0,  0,  1,  2,  3,  4,  5,  6,  7,  7,  7,  7,  8,  8,  8,  8,
	 9,  9,  9,  9, 10, 10, 10, 10, 11, 11, 11, 11, 12, 12, 12, 12,
	13, 13, 13, 13, 14, 15, 15, 15, 15, 16, 16, 16, 16, 17, 18, 18,
	18, 18, 19, 19, 19, 19, 20, 20, 20, 20, 21, 21, 21, 21, 22, 22,
	22, 22, 23, 23, 23, 23, 24, 24, 24, 24, 25, 25, 25, 25, 26, 26,
	26, 26,
};

static const int tbl_type_to_no[NUM_CHRTYPE] =
{
	 1,  2,  3,  4,  5,  6,  7,  8, 12, 16, 20, 24, 28, 32, 36, 37,
	41, 45, 46, 50, 54, 58, 62, 66, 70, 74, 78,
};

const char *chr_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_CHR <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_chrnameindex)
		chr_setnameindex();

	return str_getdef(tbl_chrnameindex[index]);
}

const char *chr_gettitlename(int index)
{
	const int tbl_titlenameindex[NUM_CHRTYPE] =
	{
		 889,  891,  893,  895,  896,  898,  899,  900,  901,  902,  903,  904,  905,  906,  907,  908,
		 909,  909,  910,  911,  912,  913,  914,  915,  916,  917,  918,
	};

	/* `FbN */
	if (index < 0 || NUM_CHRTYPE <= index)
	{
		assert(0);
		return "";
	}

	return str_getdef(tbl_titlenameindex[index]);
}

static void chr_setnameindex(void)
{
	int i;

	/* LN^̃CfbNXǂݍ */
	for (i = 0; i < NUM_CHR; i++)
	{
		unsigned char buf[SIZEOF_CHRDATA];

		file_read_sfc(buf, FILEPOS_CHRDATA + SIZEOF_CHRDATA * i, SIZEOF_CHRDATA);
		tbl_chrnameindex[i] = (buf[ 1] << 8) | buf[ 0];
	}

	flag_chrnameindex = 1;
}

chrdata_t *chr_readdata(void)
{
	int i, j;
	chrdata_t *buf_chrdata = NULL;

	/* LN^f[^i[obt@mۂ */
	if ((buf_chrdata = malloc(sizeof(chrdata_t) * NUM_CHR)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* LN^̃f[^ǂݍ */
	for (i = 0; i < NUM_CHR; i++)
	{
		unsigned char buf[SIZEOF_CHRDATA];

		file_read_sfc(buf, FILEPOS_CHRDATA + SIZEOF_CHRDATA * i, SIZEOF_CHRDATA);

		/* lnf[^ */
		buf_chrdata[i].name_index      = (buf[ 1] << 8) | buf[ 0];
		buf_chrdata[i].titlename_index = (buf[ 3] << 8) | buf[ 2];
		buf_chrdata[i].initial_level   = buf[ 5];

		/* ϐnf[^ */
		buf_chrdata[i].regist_blaze     = (buf[ 7] & 0x0c) >> 2;
		buf_chrdata[i].regist_firebane  = (buf[ 7] & 0x30) >> 4;
		buf_chrdata[i].regist_explodet  = (buf[ 7] & 0xc0) >> 6;
		buf_chrdata[i].regist_snowstorm = buf[ 8] & 0x03;
		buf_chrdata[i].regist_infernos  = (buf[ 8] & 0x0c) >> 2;
		buf_chrdata[i].regist_lightning = (buf[ 8] & 0x30) >> 4;
		buf_chrdata[i].regist_surround  = (buf[ 8] & 0xc0) >> 6;
		buf_chrdata[i].regist_sleep     = buf[ 9] & 0x03;
		buf_chrdata[i].regist_defeat    = (buf[ 9] & 0x0c) >> 2;
		buf_chrdata[i].regist_expell    = (buf[ 9] & 0x30) >> 4;
		buf_chrdata[i].regist_robmagic  = (buf[ 9] & 0xc0) >> 6;
		buf_chrdata[i].regist_stopspell = buf[10] & 0x03;
		buf_chrdata[i].regist_chaos     = (buf[10] & 0x0c) >> 2;
		buf_chrdata[i].regist_defense   = (buf[10] & 0x30) >> 4;
		buf_chrdata[i].regist_poison    = buf[11] & 0x03;
		buf_chrdata[i].regist_stun      = (buf[11] & 0x0c) >> 2;
		buf_chrdata[i].regist_stopdance = (buf[11] & 0x30) >> 4;
		buf_chrdata[i].regist_fire      = (buf[11] & 0xc0) >> 6;
		buf_chrdata[i].regist_ice       = buf[12] & 0x03;
		buf_chrdata[i].regist_rock      = (buf[12] & 0x0c) >> 2;
		buf_chrdata[i].regist_paralysis = (buf[12] & 0x30) >> 4;
		buf_chrdata[i].regist_army      = (buf[12] & 0xc0) >> 6;

		/* ̑̃f[^ */
		buf_chrdata[i].grow_pattern = buf[ 6];

		/* if[^ */
		for (j = 0; j < 5; j++)
			buf_chrdata[i].equipment_no[j] = buf[j + 16];
	}

	return buf_chrdata;
}

chrlvup_t *chr_readlvup(void)
{
	int i, j, k;
	chrlvup_t *buf_chrlvup = NULL;

	/* LN^̐f[^i[obt@mۂ */
	if ((buf_chrlvup = malloc(sizeof(chrlvup_t) * NUM_CHRTYPE)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* LN^̐f[^ǂݍ */
	for (i = 0; i < NUM_CHRTYPE; i++)
	{
		unsigned char buf[SIZEOF_CHRLVUP];
		long next_experience_point;
		int tbl_level[8];
		int tbl_ratio[7];
		struct chrlvupparam_tag tbl_param[8];

		file_read_sfc(buf, FILEPOS_CHRLVUP + SIZEOF_CHRLVUP * i, SIZEOF_CHRLVUP);

		/* ő僌x */
		buf_chrlvup[i].level_max = buf[176];
		if (buf_chrlvup[i].level_max > NUM_CHRLV)
			break;

		/* Kvoľf[^ */
		for (j = 0; j < 7; j++)
			tbl_ratio[j] = buf[j + 9];

		/* xf[^ */
		tbl_level[0] = 1;
		tbl_level[7] = buf_chrlvup[i].level_max;
		for (j = 0; j < 6; j++)
		{
			tbl_level[j + 1] = buf[j + 3];
			if (tbl_level[j + 1] > buf_chrlvup[i].level_max)
				tbl_level[j + 1] = buf_chrlvup[i].level_max;
		}

		/* p[^f[^ */
		for (j = 0; j < 8; j++)
		{
			tbl_param[j].hit_point    = (buf[j * 2 +  17] << 8) | buf[j * 2 +  16];
			tbl_param[j].magic_power  = (buf[j * 2 +  33] << 8) | buf[j * 2 +  32];
			tbl_param[j].strength     = (buf[j * 2 +  49] << 8) | buf[j * 2 +  48];
			tbl_param[j].agility      = (buf[j * 2 +  65] << 8) | buf[j * 2 +  64];
			tbl_param[j].intelligence = (buf[j * 2 +  81] << 8) | buf[j * 2 +  80];
			tbl_param[j].defense      = (buf[j * 2 +  97] << 8) | buf[j * 2 +  96];
			tbl_param[j].style        = (buf[j * 2 + 113] << 8) | buf[j * 2 + 112];
		}

		/* f[^ */
		next_experience_point = (buf[ 2] << 16) | (buf[ 1] << 8) | buf[ 0];
		for (j = 0; j < buf_chrlvup[i].level_max; j++)
		{
			int index;
			int lv_diff;
			struct chrlvupparam_tag *ptr_param1 = NULL, *ptr_param2 = NULL;

			/* Ƃp[^߂ */
			index = 0;
			for (k = 0; k < 8; k++)
			{
				if (j + 1 >= tbl_level[k])
					index++;
			}

			ptr_param1 = &tbl_param[index - 1];
			ptr_param2 = &tbl_param[index];
			lv_diff = tbl_level[index] - tbl_level[index - 1];

			/* ͈̓`FbN */
			if (lv_diff == 0)
				break;

			/* xɑΉp[^Zo */
			buf_chrlvup[i].level[j].hit_point =
				ptr_param1->hit_point    + (ptr_param2->hit_point    - ptr_param1->hit_point)    * (j + 1 - tbl_level[index - 1]) / lv_diff;
			buf_chrlvup[i].level[j].magic_power =
				ptr_param1->magic_power  + (ptr_param2->magic_power  - ptr_param1->magic_power)  * (j + 1 - tbl_level[index - 1]) / lv_diff;
			buf_chrlvup[i].level[j].strength =
				ptr_param1->strength     + (ptr_param2->strength     - ptr_param1->strength)     * (j + 1 - tbl_level[index - 1]) / lv_diff;
			buf_chrlvup[i].level[j].agility =
				ptr_param1->agility      + (ptr_param2->agility      - ptr_param1->agility)      * (j + 1 - tbl_level[index - 1]) / lv_diff;
			buf_chrlvup[i].level[j].defense =
				ptr_param1->defense      + (ptr_param2->defense      - ptr_param1->defense)      * (j + 1 - tbl_level[index - 1]) / lv_diff;
			buf_chrlvup[i].level[j].intelligence =
				ptr_param1->intelligence + (ptr_param2->intelligence - ptr_param1->intelligence) * (j + 1 - tbl_level[index - 1]) / lv_diff;
			buf_chrlvup[i].level[j].style =
				ptr_param1->style        + (ptr_param2->style        - ptr_param1->style)        * (j + 1 - tbl_level[index - 1]) / lv_diff;

			/* KvolZo */
			if (j == 0)
				buf_chrlvup[i].level[j].experience_point = 0;
			else
			{
				buf_chrlvup[i].level[j].experience_point = buf_chrlvup[i].level[j - 1].experience_point + next_experience_point;
				next_experience_point = next_experience_point * tbl_ratio[index - 1] / 32;
			}
		}
		for (; j < NUM_CHRLV; j++)
		{
			buf_chrlvup[i].level[j].experience_point = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].experience_point;
			buf_chrlvup[i].level[j].hit_point        = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].hit_point;
			buf_chrlvup[i].level[j].magic_power      = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].magic_power;
			buf_chrlvup[i].level[j].strength         = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].strength;
			buf_chrlvup[i].level[j].agility          = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].agility;
			buf_chrlvup[i].level[j].defense          = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].defense;
			buf_chrlvup[i].level[j].intelligence     = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].intelligence;
			buf_chrlvup[i].level[j].style            = buf_chrlvup[i].level[buf_chrlvup[i].level_max - 1].style;
		}
	}

	return buf_chrlvup;
}

int chr_writeregist(const char *ptr_filename, const chrdata_t *ptr_chrdata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_chrdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* LN^̑ϐnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* LN^̑ϐnf[^o͂ */
	fprintf(fp, "No.      O       "
				"Ή M  C ^ d f Ö V ގU MPz   猸     x       R\n");
	for (i = 0; i < NUM_CHR; i++)
	{
		fprintf(fp, "%03d(%02X): %-8s :  %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\n", i, i,
				chr_getname(i),
				ptr_chrdata[i].regist_blaze,
				ptr_chrdata[i].regist_firebane,
				ptr_chrdata[i].regist_explodet,
				ptr_chrdata[i].regist_snowstorm,
				ptr_chrdata[i].regist_infernos,
				ptr_chrdata[i].regist_lightning,
				ptr_chrdata[i].regist_surround,
				ptr_chrdata[i].regist_sleep,
				ptr_chrdata[i].regist_defeat,
				ptr_chrdata[i].regist_expell,
				ptr_chrdata[i].regist_robmagic,
				ptr_chrdata[i].regist_stopspell,
				ptr_chrdata[i].regist_chaos,
				ptr_chrdata[i].regist_defense,
				ptr_chrdata[i].regist_poison,
				ptr_chrdata[i].regist_stun,
				ptr_chrdata[i].regist_stopdance,
				ptr_chrdata[i].regist_fire,
				ptr_chrdata[i].regist_ice,
				ptr_chrdata[i].regist_rock,
				ptr_chrdata[i].regist_paralysis,
				ptr_chrdata[i].regist_army);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int chr_writeequip(const char *ptr_filename, const chrdata_t *ptr_chrdata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "------------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_chrdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* LN^̑if[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* LN^̑if[^o͂ */
	fprintf(fp, "No.      O       "
				"                "
				"Z                  "
				"                  "
				"                  "
				"i\n");
	for (i = 0; i < NUM_CHR; i++)
	{
		const char *ptr_name[5];

		for (j = 0; j < 5; j++)
			ptr_name[j] = (ptr_chrdata[i].equipment_no[j] != 0x00) ? item_getname(ptr_chrdata[i].equipment_no[j]) : str_nothing;

		fprintf(fp, "%03d(%02X): %-8s : ", i, i, chr_getname(i));
		for (j = 0; j < 5; j++)
		{
			if (j != 0)
				fprintf(fp, ", ");
			fprintf(fp, "%-18s", ptr_name[j]);
		}
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int chr_writelvup(const char *ptr_filename, const chrlvup_t *ptr_chrlvup)
{
	int i, j;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_chrlvup == NULL)
	{
		assert(0);
		return 0;
	}

	/* LN^f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* LN^f[^o͂ */
	for (i = 0; i < NUM_CHRTYPE; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%03d(%02X): %s(%s)]\n", i, i,
				chr_getname(tbl_type_to_no[i]),
				chr_gettitlename(i));

		/* f[^ */
		fprintf(fp, "LV  őHP őMP "
					"              i    ol\n");
		for (j = 0; j < ptr_chrlvup[i].level_max; j++)
		{
			fprintf(fp, "%2d:  %5d, %5d, %3d, %3d, %3d, %3d, %3d, %8ld\n", j + 1,
					ptr_chrlvup[i].level[j].hit_point,
					ptr_chrlvup[i].level[j].magic_power,
					ptr_chrlvup[i].level[j].strength,
					ptr_chrlvup[i].level[j].agility,
					ptr_chrlvup[i].level[j].defense,
					ptr_chrlvup[i].level[j].intelligence,
					ptr_chrlvup[i].level[j].style,
					ptr_chrlvup[i].level[j].experience_point);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
