/*
 *	dq6item.c
 *		Copyright (C)2001-2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "dq6define.h"
#include "dq6item.h"

#define FILEPOS_ITEMDATA	0x04003d
#define NUM_ITEM			256
#define SIZEOF_ITEMDATA		26

static void item_setnameindex(void);

static int tbl_itemnameindex[NUM_ITEM];	/* \̃CfbNXi[e[u */
static int flag_itemnameindex = 0;		/* tbl_itemnameindexɃf[^i[Ă邩ǂ */

const char *item_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_ITEM <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_itemnameindex)
		item_setnameindex();

	return str_getdef(tbl_itemnameindex[index]);
}

static void item_setnameindex(void)
{
	int i;

	/* ̖Of[^ǂݍ */
	for (i = 0; i < NUM_ITEM; i++)
	{
		unsigned char buf[SIZEOF_ITEMDATA];

		file_read_sfc(buf, FILEPOS_ITEMDATA + SIZEOF_ITEMDATA * i, SIZEOF_ITEMDATA);
		tbl_itemnameindex[i] = (buf[ 1] << 8) | buf[ 0];
	}

	flag_itemnameindex = 1;
}

itemdata_t *item_readdata(void)
{
	int i;
	itemdata_t *buf_itemdata = NULL;

	/* ̊ef[^i[obt@mۂ */
	if ((buf_itemdata = malloc(sizeof(itemdata_t) * NUM_ITEM)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* ̊ef[^ǂݍ */
	for (i = 0; i < NUM_ITEM; i++)
	{
		unsigned char buf[SIZEOF_ITEMDATA];

		file_read_sfc(buf, FILEPOS_ITEMDATA + SIZEOF_ITEMDATA * i, SIZEOF_ITEMDATA);

		/* lnf[^ */
		buf_itemdata[i].name_index = (buf[ 1] << 8) | buf[ 0];

		buf_itemdata[i].price          = (buf[ 4] << 16) | (buf[ 3] << 8) | buf[ 2];
		buf_itemdata[i].various_value  = (buf[25] << 16) | (buf[24] << 8) | buf[23];
		buf_itemdata[i].increase_value = (buf[ 8] << 8) | buf[ 7];
		buf_itemdata[i].increase_param = buf[ 6];
		buf_itemdata[i].increase_style = (buf[10] << 8) | buf[ 9];
		if ((buf[10] & 0x80) != 0x00)
			buf_itemdata[i].increase_style -= 0x10000;
	}

	return buf_itemdata;
}

int item_writeparam(const char *ptr_filename, const itemdata_t *ptr_itemdata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_itemdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* ̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* ̐lnf[^o͂ */
	fprintf(fp, "No.      O                 "
				"  i      l U  f \n");
	for (i = 0; i < NUM_ITEM; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s : %6ld, %6ld, %5d, %5d, %5d, %5d\n", i, i,
				item_getname(i),
				ptr_itemdata[i].price,
				ptr_itemdata[i].various_value,
				(ptr_itemdata[i].increase_param == 0x01) ? ptr_itemdata[i].increase_value : 0,
				(ptr_itemdata[i].increase_param == 0x03) ? ptr_itemdata[i].increase_value : 0,
				(ptr_itemdata[i].increase_param == 0x02) ? ptr_itemdata[i].increase_value : 0,
				ptr_itemdata[i].increase_style);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
