/*
 *	dq6monster.c
 *		Copyright (C)2001-2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "dq6define.h"
#include "dq6item.h"
#include "dq6spell.h"
#include "dq6monster.h"

#define FILEPOS_MONDATA		0x020154
#define FILEPOS_MONENC		0x06843c
#define FILEPOS_MONMAP		0x089138
#define NUM_MON				256
#define NUM_MONENC			117
#define NUM_MONMAP			6
#define SIZEOF_MONDATA		42
#define SIZEOF_MONENC		22

static void mon_setnameindex(void);

static int tbl_monnameindex[NUM_MON];	/* X^[\̃CfbNXi[e[u */
static int flag_monnameindex = 0;		/* tbl_monnameindexɃf[^i[Ă邩ǂ */

const char *mon_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_MON <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_monnameindex)
		mon_setnameindex();

	return str_getdef(tbl_monnameindex[index]);
}

static void mon_setnameindex(void)
{
	int i;

	/* X^[̃CfbNXǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONDATA];

		file_read_sfc(buf, FILEPOS_MONDATA + SIZEOF_MONDATA * i, SIZEOF_MONDATA);
		tbl_monnameindex[i] = (buf[41] << 8) | buf[40];
	}

	flag_monnameindex = 1;
}

mondata_t *mon_readdata(void)
{
	int i, j;
	mondata_t *buf_mondata = NULL;

	/* X^[̊ef[^i[obt@mۂ */
	if ((buf_mondata = malloc(sizeof(mondata_t) * NUM_MON)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[̊ef[^ǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONDATA];

		file_read_sfc(buf, FILEPOS_MONDATA + SIZEOF_MONDATA * i, SIZEOF_MONDATA);

		/* lnf[^ */
		buf_mondata[i].name_index       = (buf[41] << 8) | buf[40];
		buf_mondata[i].level            = (buf[13] & 0xfe) >> 1;
		buf_mondata[i].hit_point        = (buf[37] << 8) | buf[36];
		buf_mondata[i].magic_power      = buf[31];
		buf_mondata[i].experience_point = (buf[39] << 8) | buf[38];
		buf_mondata[i].gold             = (buf[19] << 4) | ((buf[18] & 0xf0) >> 4);
		buf_mondata[i].attack_power     = ((buf[15] & 0x03) << 8) | buf[14];
		buf_mondata[i].defense_power    = ((buf[17] & 0x03) << 8) | buf[16];
		buf_mondata[i].agility          = ((buf[13] & 0x01) << 8) | buf[12];

		/* ϐnf[^ */
		buf_mondata[i].regist_blaze     = (buf[ 1] & 0x0c) >> 2;
		buf_mondata[i].regist_firebane  = (buf[ 3] & 0x0c) >> 2;
		buf_mondata[i].regist_explodet  = (buf[ 5] & 0x0c) >> 2;
		buf_mondata[i].regist_snowstorm = (buf[ 7] & 0x0c) >> 2;
		buf_mondata[i].regist_infernos  = (buf[ 9] & 0x0c) >> 2;
		buf_mondata[i].regist_lightning = (buf[11] & 0x0c) >> 2;
		buf_mondata[i].regist_surround  = buf[18] & 0x03;
		buf_mondata[i].regist_sleep     = (buf[18] & 0x0c) >> 2;
		buf_mondata[i].regist_defeat    = (buf[21] & 0x06) >> 1;
		buf_mondata[i].regist_expell    = (buf[21] & 0x18) >> 3;
		buf_mondata[i].regist_robmagic  = (buf[21] & 0x60) >> 5;
		buf_mondata[i].regist_stopspell = (buf[23] & 0x06) >> 1;
		buf_mondata[i].regist_chaos     = (buf[23] & 0x18) >> 3;
		buf_mondata[i].regist_defense   = (buf[23] & 0x60) >> 5;
		buf_mondata[i].regist_poison    = (buf[24] & 0x0c) >> 2;
		buf_mondata[i].regist_stun      = (buf[24] & 0x30) >> 4;
		buf_mondata[i].regist_stopdance = (buf[24] & 0xc0) >> 6;
		buf_mondata[i].regist_fire      = buf[25] & 0x03;
		buf_mondata[i].regist_ice       = (buf[25] & 0x0c) >> 2;
		buf_mondata[i].regist_rock      = (buf[25] & 0x30) >> 4;
		buf_mondata[i].regist_paralysis  = (buf[25] & 0xc0) >> 6;
		buf_mondata[i].regist_army      = buf[26] & 0x03;

		/* tOnf[^ */
		buf_mondata[i].flag_type_dragon = ((buf[23] & 0x80) != 0x00);
		buf_mondata[i].flag_type_zombie = ((buf[28] & 0x10) != 0x00);
		buf_mondata[i].flag_type_metal  = ((buf[21] & 0x80) != 0x00);
		buf_mondata[i].flag_type_slime  = ((buf[28] & 0x40) != 0x00);
		buf_mondata[i].flag_type_frying = ((buf[28] & 0x20) != 0x00);
		buf_mondata[i].flag_intensive   = ((buf[29] & 0x80) != 0x00);

		/* ̑̃f[^ */
		buf_mondata[i].chr_join          = (buf[17] & 0xe0) >> 5;
		buf_mondata[i].item_no           = buf[32];
		buf_mondata[i].item_get          = (buf[15] & 0x1c) >> 2;
		buf_mondata[i].evade_rate        = (buf[17] & 0x1c) >> 2;
		buf_mondata[i].auto_healing      = (buf[29] & 0x30) >> 4;
		buf_mondata[i].repeatedly_action = (buf[29] & 0x0c) >> 2;
		buf_mondata[i].apriori_statecng  = ((buf[34] & 0x04) == 0x00) ? 0 : 1;
		buf_mondata[i].rate_statecng     = (buf[28] & 0x0c) >> 2;
		buf_mondata[i].action_type       = (buf[15] & 0xe0) >> 5;
		buf_mondata[i].judgment          = buf[29] & 0x03;

		/* sp^[f[^ */
		for (j = 0; j < 6; j++)
			buf_mondata[i].action[j] = ((buf[j * 2 + 1] & 0x01) << 8) | buf[j * 2];
	}

	return buf_mondata;
}


monenc_t *mon_readenc(void)
{
	int i, j;
	monenc_t *buf_monenc = NULL;

	/* X^[e[uf[^i[obt@mۂ */
	if ((buf_monenc = malloc(sizeof(monenc_t) * NUM_MONENC)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[e[uf[^ǂݍ */
	for (i = 0; i < NUM_MONENC; i++)
	{
		unsigned char buf[SIZEOF_MONENC];
		unsigned int word[10];

		file_read_sfc(buf, FILEPOS_MONENC + SIZEOF_MONENC * i, SIZEOF_MONENC);

		buf_monenc[i].level = buf[0];

		word[0] = ((buf[ 5] & 0x07) << 11) | ((buf[ 4] & 0xff) << 3) | ((buf[ 3] & 0xe0) >> 5);
		word[1] = ((buf[ 7] & 0x01) << 13) | ((buf[ 6] & 0xff) << 5) | ((buf[ 5] & 0xf8) >> 3);
		word[2] =                            ((buf[ 8] & 0x7f) << 7) | ((buf[ 7] & 0xfe) >> 1);
		word[3] = ((buf[10] & 0x1f) <<  9) | ((buf[ 9] & 0xff) << 1) | ((buf[ 8] & 0x80) >> 7);
		word[4] = ((buf[12] & 0x07) << 11) | ((buf[11] & 0xff) << 3) | ((buf[10] & 0xe0) >> 5);
		word[5] = ((buf[14] & 0x01) << 13) | ((buf[13] & 0xff) << 5) | ((buf[12] & 0xf8) >> 3);
		word[6] =                            ((buf[15] & 0x7f) << 7) | ((buf[14] & 0xfe) >> 1);
		word[7] = ((buf[17] & 0x1f) <<  9) | ((buf[16] & 0xff) << 1) | ((buf[15] & 0x80) >> 7);
		word[8] = ((buf[19] & 0x07) << 11) | ((buf[18] & 0xff) << 3) | ((buf[17] & 0xe0) >> 5);
		word[9] = ((buf[21] & 0x01) << 13) | ((buf[20] & 0xff) << 5) | ((buf[19] & 0xf8) >> 3);

		for (j = 0; j < 10; j++)
		{
			buf_monenc[i].mon[j].monster_no = word[j] & 0x00ff;
			buf_monenc[i].mon[j].unknown    = (word[j] & 0x3f00) >> 8;
		}
	}

	return buf_monenc;
}

monmap_t *mon_readmap(void)
{
	int i, x, y;
	unsigned char *buf = NULL;
	unsigned char *ptr_buf = NULL;
	monmap_t *buf_monmap = NULL;
	const size_t bufsize = 1536;

	/* X^[}bvf[^i[obt@mۂ */
	if ((buf_monmap = malloc(sizeof(monmap_t) * NUM_MONMAP)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[}bvf[^ǂݍ */
	if ((buf = malloc(bufsize)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);
	memset(buf, 0x00, bufsize);
	file_read_sfc(buf, FILEPOS_MONMAP, bufsize);

	ptr_buf = buf;
	for (i = 0; i < NUM_MONMAP; i++)
	{
		for (y = 0; y < 16; y++)
		{
			for (x = 0; x < 16; x++)
				buf_monmap[i].area[x][y] = *ptr_buf++;
		}
	}

	free(buf);
	buf = NULL;

	return buf_monmap;
}

int mon_writeparam(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̐lnf[^o͂ */
	fprintf(fp, "No.      O                 "
				"  Lv     HP     MP     Ex      G "
				"U  f\n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s :  %3d, %5d, %5d, %5d, %5d, %5d, %5d, %5d\n", i, i,
				mon_getname(i),
				ptr_mondata[i].level,
				ptr_mondata[i].hit_point,
				ptr_mondata[i].magic_power,
				ptr_mondata[i].experience_point,
				ptr_mondata[i].gold,
				ptr_mondata[i].attack_power,
				ptr_mondata[i].defense_power,
				ptr_mondata[i].agility);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeregist(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̑ϐnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̑ϐnf[^o͂ */
	fprintf(fp, "No.      O                 "
				"Ή M  C ^ d f Ö V ގU MPz   猸     x       R\n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s :  %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\n", i, i,
				mon_getname(i),
				ptr_mondata[i].regist_blaze,
				ptr_mondata[i].regist_firebane,
				ptr_mondata[i].regist_explodet,
				ptr_mondata[i].regist_snowstorm,
				ptr_mondata[i].regist_infernos,
				ptr_mondata[i].regist_lightning,
				ptr_mondata[i].regist_surround,
				ptr_mondata[i].regist_sleep,
				ptr_mondata[i].regist_defeat,
				ptr_mondata[i].regist_expell,
				ptr_mondata[i].regist_robmagic,
				ptr_mondata[i].regist_stopspell,
				ptr_mondata[i].regist_chaos,
				ptr_mondata[i].regist_defense,
				ptr_mondata[i].regist_poison,
				ptr_mondata[i].regist_stun,
				ptr_mondata[i].regist_stopdance,
				ptr_mondata[i].regist_fire,
				ptr_mondata[i].regist_ice,
				ptr_mondata[i].regist_rock,
				ptr_mondata[i].regist_paralysis,
				ptr_mondata[i].regist_army);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeitem(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;
	const char *str_nothing = "------------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̕󔠂̓f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̕󔠂̓f[^o͂ */
	fprintf(fp, "No.      O                 "
				"x   \n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%02X): %-18s :  %3d,  %-18s\n", i, i,
				mon_getname(i),
				ptr_mondata[i].item_get,
				(ptr_mondata[i].item_no != 0x00) ? item_getname(ptr_mondata[i].item_no) : str_nothing);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeact(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_repeat[] =
	{
		"1",
		"1`2",
		"2",
		"3",
	};

	assert(sizeof(str_repeat) / sizeof(str_repeat[0]) == 4);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̍sp^[f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̍sp^[f[^o͂ */
	for (i = 0; i < NUM_MON; i++)
	{
		if (ptr_mondata[i].repeatedly_action >= 4)
			break;

		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%03d(%02X): %s]\n", i, i, mon_getname(i));
		fprintf(fp, "s:   %s\n", str_repeat[ptr_mondata[i].repeatedly_action]);

		fprintf(fp, "No. s\n");
		for (j = 0; j < 6; j++)
		{
			const char *ptr_spellname = spell_getname(ptr_mondata[i].action[j]);
			const char *ptr_spelldesc = spell_getdesc(ptr_mondata[i].action[j]);

			fprintf(fp, "%02d:", j + 1);
			if (strlen(ptr_spellname) > 0)
				fprintf(fp, " %s", ptr_spellname);
			if (strlen(ptr_spelldesc) > 0)
				fprintf(fp, " (%s)", ptr_spelldesc);
			fputc('\n', fp);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writemisc(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;
	const char *str_state[] = {"", "}zJ^"};

	assert(sizeof(str_state) / sizeof(str_state[0]) == 2);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̂̑̃f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̂̑̃f[^o͂ */
	fprintf(fp, "No.      O                 "
				"   s    W    f ԕω\n");
	for (i = 0; i < NUM_MON; i++)
	{
		if (ptr_mondata[i].apriori_statecng >= 2)
			break;

		fprintf(fp, "%03d(%02X): %-18s :  %2s   %2s   %2s   %2s   %2s   %2s   %3d, %3d, %3d, %3d, %3d", i, i,
				mon_getname(i),
				str_getflag(ptr_mondata[i].flag_type_dragon),
				str_getflag(ptr_mondata[i].flag_type_zombie),
				str_getflag(ptr_mondata[i].flag_type_metal),
				str_getflag(ptr_mondata[i].flag_type_slime),
				str_getflag(ptr_mondata[i].flag_type_frying),
				str_getflag(ptr_mondata[i].flag_intensive),
				ptr_mondata[i].chr_join,
				ptr_mondata[i].evade_rate,
				ptr_mondata[i].auto_healing,
				ptr_mondata[i].judgment,
				ptr_mondata[i].rate_statecng);
		if (ptr_mondata[i].rate_statecng > 0)
			fprintf(fp, ", %s", str_state[ptr_mondata[i].apriori_statecng]);
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeenc(const char *ptr_filename, const monenc_t *ptr_monenc)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "------------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_monenc == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[e[uf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[e[uo͂ */
	for (i = 0; i < NUM_MONENC; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[%03d(%02X)]\n", i, i);
		fprintf(fp, "x: %3d\n", ptr_monenc[i].level);
		fprintf(fp, "No. X^[\n");
		for (j = 0; j < 10; j++)
		{
			fprintf(fp, "%02d: %-18s\n", j,
					(ptr_monenc[i].mon[j].monster_no != 0) ? mon_getname(ptr_monenc[i].mon[j].monster_no) : str_nothing);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writemap(const char *ptr_filename, const monmap_t *ptr_monmap)
{
	int i, x, y;
	FILE *fp = NULL;
	const char *str_name[/* NUM_MONMAP */] =
	{
		"̐E",
		"̐E",
		"̐E(C)",
		"͂܂̐E",
		"̐E(C)",
		"̐E(C)",
	};

	assert(sizeof(str_name) / sizeof(str_name[0]) == NUM_MONMAP);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_monmap == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[}bvf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[}bvo͂ */
	for (i = 0; i < NUM_MONMAP; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%1d: %s]\n", i, str_name[i]);
		for (y = 0; y < 16; y++)
		{
			for (x = 0; x < 16; x++)
			{
				if (x != 0)
					fprintf(fp, " ");

				if (ptr_monmap[i].area[x][y] != 0x00)
					fprintf(fp, "%03d", ptr_monmap[i].area[x][y]);
				else
					fprintf(fp, "---");
			}
			fputc('\n', fp);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
