/*
 *	ff7savedata.c
 *		Copyright (C)2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "ff7decode.h"
#include "ff7savedata.h"

#define MCPOS_TITLE			0x000004
#define MCPOS_CHRDATA		0x000254
#define MCPOS_LOVETALK		0x000da7
#define MCPOS_LOVEBATTLE	0x000df4
#define SIZEOF_CHRDATA		132
#define SIZEOF_CHRNAME		12

savedata_t *sav_readdata(const unsigned char *ptr_buf)
{
	int i;
	const unsigned char *ptr = NULL;
	savedata_t *buf_savedata = NULL;

	/* `FbN */
	if (ptr_buf == NULL)
	{
		assert(0);
		return NULL;
	}

	/* Z[uf[^͌ʂi[obt@mۂ */
	if ((buf_savedata = malloc(sizeof(savedata_t))) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* Z[uf[^̃^Cgǂݍ */
	ptr = ptr_buf + MCPOS_TITLE;
	memset(buf_savedata->title, '\0', sizeof(buf_savedata->title));
	memmove(buf_savedata->title, ptr, sizeof(buf_savedata->title) - 1);

	/* eLN^̃Xe[^Xǂݍ */
	ptr = ptr_buf + MCPOS_CHRDATA;
	for (i = 0; i < NUM_CHR; i++)
	{
		size_t len = SIZEOF_CHRNAME;
		const unsigned char *ptr_tail = memchr(&ptr[16], 0xff, SIZEOF_CHRNAME);

		if (ptr_tail != NULL)
			len = (size_t)(ptr_tail - &ptr[16]);

		/* O */
		dec_string(buf_savedata->chr[i].name, &ptr[16], sizeof(buf_savedata->chr[i].name), len, 0);

		/* lnf[^ */
		buf_savedata->chr[i].level            = ptr[ 1];
		buf_savedata->chr[i].hit_point        = ((long)ptr[45] << 8) + ptr[44];
		buf_savedata->chr[i].hit_point_max    = ((long)ptr[47] << 8) + ptr[46];
		buf_savedata->chr[i].hit_point_cng    = ((long)ptr[57] << 8) + ptr[56];
		buf_savedata->chr[i].magic_point      = ((long)ptr[49] << 8) + ptr[48];
		buf_savedata->chr[i].magic_point_max  = ((long)ptr[51] << 8) + ptr[50];
		buf_savedata->chr[i].magic_point_cng  = ((long)ptr[59] << 8) + ptr[58];
		buf_savedata->chr[i].experience_point = ((long)ptr[63] << 24) + ((long)ptr[62] << 16) + ((long)ptr[61] << 8) + ptr[60];
		buf_savedata->chr[i].strength         = ptr[ 2];
		buf_savedata->chr[i].speed            = ptr[ 6];
		buf_savedata->chr[i].stamina          = ptr[ 3];
		buf_savedata->chr[i].magic_power      = ptr[ 4];
		buf_savedata->chr[i].will_power       = ptr[ 5];
		buf_savedata->chr[i].luck             = ptr[ 6];

		ptr += SIZEOF_CHRDATA;
	}

	/* Dxǂݍ */
	ptr = ptr_buf + MCPOS_LOVETALK;
	for (i = 0; i < NUM_CHRLOVE; i++)
		buf_savedata->love_talk[i] = ptr[i];
	ptr = ptr_buf + MCPOS_LOVEBATTLE;
	for (i = 0; i < NUM_CHRLOVE; i++)
		buf_savedata->love_btl[i] = ptr[i];

	return buf_savedata;
}

int sav_writetitle(FILE *stream, const savedata_t *ptr_savedata)
{
	/* `FbN */
	if (stream == NULL || ptr_savedata == NULL)
	{
		assert(0);
		return 0;
	}

	/* [J[h̃^Cgo͂(Shift_JISŒ) */
	fputs("[", stream);
	fputs(ptr_savedata->title, stream);
	fputs("]\n", stream);

	return 1;
}

int sav_writestatus(FILE *stream, const savedata_t *ptr_savedata)
{
	int i;

	/* `FbN */
	if (stream == NULL || ptr_savedata == NULL)
	{
		assert(0);
		return 0;
	}

	/* eLN^̃Xe[^Xo͂ */
	fprintf(stream, "O           "
					" LV     HP     MP        EXP "
					"      ̗  _   ^\n");
	for (i = 0; i < NUM_CHR; i++)
	{
		fprintf(stream, "%-12s : %3d, %5ld, %5ld,%10ld, %3d, %3d, %3d, %3d, %3d, %3d\n",
				ptr_savedata->chr[i].name,
				ptr_savedata->chr[i].level,
				ptr_savedata->chr[i].hit_point,
				ptr_savedata->chr[i].magic_point,
				ptr_savedata->chr[i].experience_point,
				ptr_savedata->chr[i].strength,
				ptr_savedata->chr[i].speed,
				ptr_savedata->chr[i].stamina,
				ptr_savedata->chr[i].magic_power,
				ptr_savedata->chr[i].will_power,
				ptr_savedata->chr[i].luck);
	}

	return 1;
}

int sav_writelove(FILE *stream, const savedata_t *ptr_savedata)
{
	int i;
	const int tbl_convert[NUM_CHRLOVE] =
	{
		3,	/* GAX */
		2,	/* eBt@ */
		5,	/* tB */
		1,	/* obg */
	};

	assert(sizeof(tbl_convert) / sizeof(tbl_convert[0]) == NUM_CHRLOVE);

	/* `FbN */
	if (stream == NULL || ptr_savedata == NULL)
	{
		assert(0);
		return 0;
	}

	/* Dxo͂ */
	fprintf(stream, "O           "
					"b 퓬\n");
	for (i = 0; i < NUM_CHRLOVE; i++)
	{
		fprintf(stream, "%-12s :  %3d, %3d\n",
				ptr_savedata->chr[tbl_convert[i]].name,
				ptr_savedata->love_talk[i],
				ptr_savedata->love_btl[i]);
	}

	return 1;
}

