/*
 *	rs2item.c
 *		Copyright (C)2002, 2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "rs2decode.h"
#include "rs2tech.h"
#include "rs2item.h"

#define FILEPOS_ITEMDATA			0x1e0000
#define FILEPOS_ITEMDATA2			0x1e06c0
#define FILEPOS_ITEMNAME			0x1dba00
#define FILEPOS_ITEMPRICE			0x1d86b0
#define NUM_ITEM_WEAPON				96
#define NUM_ITEM_PROTECTOR			96
#define NUM_ITEM_SHIELD				16
#define NUM_ITEM_OTHER				48
#define NUM_ITEM					(NUM_ITEM_WEAPON + NUM_ITEM_PROTECTOR + NUM_ITEM_SHIELD + NUM_ITEM_OTHER)	/* = 256 */
#define SIZEOF_ITEMDATA_WEAPON		4
#define SIZEOF_ITEMDATA_PROTECTOR	12
#define SIZEOF_ITEMDATA_SHIELD		3
#define SIZEOF_ITEMDATA2			6
#define SIZEOF_ITEMNAME				10
#define SIZEOF_ITEMPRICE			2
#define REGION_WEAPON_BEGIN			0
#define REGION_WEAPON_END			(REGION_WEAPON_BEGIN + NUM_ITEM_WEAPON - 1)			/* =  95 */
#define REGION_PROTECTOR_BEGIN		(REGION_WEAPON_END + 1)								/* =  96 */
#define REGION_PROTECTOR_END		(REGION_PROTECTOR_BEGIN + NUM_ITEM_PROTECTOR - 1)	/* = 191 */
#define REGION_SHIELD_BEGIN			(REGION_PROTECTOR_END + 1)							/* = 192 */
#define REGION_SHIELD_END			(REGION_SHIELD_BEGIN + NUM_ITEM_SHIELD - 1)			/* = 207 */
#define REGION_OTHER_BEGIN			(REGION_SHIELD_END + 1)								/* = 208 */
#define REGION_OTHER_END			(REGION_OTHER_BEGIN + NUM_ITEM_OTHER - 1)			/* = 255 */

static void item_setname(void);

static char tbl_itemname[NUM_ITEM][256];	/* ACe̖Oi[e[u */
static int flag_setitemname = 0;			/* tbl_itemnameɃf[^i[Ă邩ǂ */

const char *item_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_ITEM <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setitemname)
		item_setname();

	return tbl_itemname[index];
}

static void item_setname(void)
{
	int i;

	/* ACe̖Of[^ */
	for (i = 0; i < NUM_ITEM; i++)
		*tbl_itemname[i] = '\0';

	/* ACe̖Of[^ǂݍ */
	for (i = 0; i < NUM_ITEM; i++)
	{
		unsigned char buf[SIZEOF_ITEMNAME];

		file_read_sfc(buf, FILEPOS_ITEMNAME + SIZEOF_ITEMNAME * i, SIZEOF_ITEMNAME);
		dec_string(tbl_itemname[i], buf, sizeof(tbl_itemname[i]), SIZEOF_ITEMNAME);
	}

	flag_setitemname = 1;
}

itemdata_t *item_readdata(void)
{
	int i, j;
	long filepos;
	itemdata_t *buf_itemdata = NULL;

	/* ACe̊ef[^i[obt@mۂ */
	if ((buf_itemdata = malloc(sizeof(itemdata_t) * NUM_ITEM)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* ACẻʕ\p̃f[^ǂݍ */
	for (i = 0; i < NUM_ITEM; i++)
	{
		unsigned char buf[SIZEOF_ITEMDATA2];

		file_read_sfc(buf, FILEPOS_ITEMDATA2 + SIZEOF_ITEMDATA2 * i, SIZEOF_ITEMDATA2);

		buf_itemdata[i].power      = buf[ 2];
		buf_itemdata[i].message_no = buf[ 0];
	}

	/* ACẻif[^ǂݍ */
	for (i = 0; i < NUM_ITEM; i++)
	{
		unsigned char buf[SIZEOF_ITEMPRICE];

		file_read_sfc(buf, FILEPOS_ITEMPRICE + SIZEOF_ITEMPRICE * i, SIZEOF_ITEMPRICE);

		buf_itemdata[i].price = (buf[1] << 8) | buf[0];
	}

	/* ACe̊ef[^ǂݍ */
	filepos = FILEPOS_ITEMDATA;
	for (i = 0; i < NUM_ITEM; i++)
	{
		/* ACe̎ނ𔻕ʂ */
		if (REGION_WEAPON_BEGIN <= i && i <= REGION_WEAPON_END)
		{
			unsigned char buf[SIZEOF_ITEMDATA_WEAPON];

			filepos += file_read_sfc(buf, filepos, SIZEOF_ITEMDATA_WEAPON);

			buf_itemdata[i].type = ITEM_TYPE_WEAPON;

			/* lnf[^ */
			buf_itemdata[i].param.weapon.attack_power = buf[ 0];
			buf_itemdata[i].param.weapon.weight       = buf[ 1] & 0x0f;

			/* ŗLZf[^ */
			for (j = 0; j < 2; j++)
				buf_itemdata[i].param.weapon.tech_no[j] = buf[j + 2];
		}
		else if (REGION_PROTECTOR_BEGIN <= i && i <= REGION_PROTECTOR_END)
		{
			unsigned char buf[SIZEOF_ITEMDATA_PROTECTOR];

			filepos += file_read_sfc(buf, filepos, SIZEOF_ITEMDATA_PROTECTOR);

			buf_itemdata[i].type = ITEM_TYPE_PROTECTOR;

			/* lnf[^ */
			buf_itemdata[i].param.protector.defense_slash   = buf[ 0];
			buf_itemdata[i].param.protector.defense_knock   = buf[ 1];
			buf_itemdata[i].param.protector.defense_poke    = buf[ 2];
			buf_itemdata[i].param.protector.defense_shoot   = buf[ 3];
			buf_itemdata[i].param.protector.defense_fire    = buf[ 4];
			buf_itemdata[i].param.protector.defense_ice     = buf[ 5];
			buf_itemdata[i].param.protector.defense_thunder = buf[ 6];
			buf_itemdata[i].param.protector.defense_state   = buf[ 7];

			buf_itemdata[i].param.protector.weight = buf[11];

			/* tOnf[^ */
			buf_itemdata[i].param.protector.regist_state = buf[ 8];

			/* p[^ωf[^ */
			buf_itemdata[i].param.protector.increase_strength =
				((buf[10] & 0x80) != 0) ? (buf[10] & 0x03) + 1 : 0;
			buf_itemdata[i].param.protector.increase_dexterity =
				((buf[10] & 0x40) != 0) ? (buf[10] & 0x03) + 1 : 0;
			buf_itemdata[i].param.protector.increase_magic_power =
				((buf[10] & 0x20) != 0) ? (buf[10] & 0x03) + 1 : 0;
			buf_itemdata[i].param.protector.increase_unknown =
				((buf[10] & 0x10) != 0) ? (buf[10] & 0x03) + 1 : 0;
			buf_itemdata[i].param.protector.increase_speed =
				((buf[10] & 0x08) != 0) ? (buf[10] & 0x03) + 1 : 0;
			buf_itemdata[i].param.protector.increase_stamina =
				((buf[10] & 0x04) != 0) ? (buf[10] & 0x03) + 1 : 0;
		}
		else if (REGION_SHIELD_BEGIN <= i && i <= REGION_SHIELD_END)
		{
			unsigned char buf[SIZEOF_ITEMDATA_SHIELD];

			filepos += file_read_sfc(buf, filepos, SIZEOF_ITEMDATA_SHIELD);

			buf_itemdata[i].type = ITEM_TYPE_SHIELD;

			/* lnf[^ */
			buf_itemdata[i].param.shield.defense_power = buf[ 1] & 0x7f;
			buf_itemdata[i].param.shield.evade_rate    = (buf[ 2] & 0xf8) >> 2;
			buf_itemdata[i].param.shield.weight        = buf[ 2] & 0x07;

			/* tOnf[^ */
			buf_itemdata[i].param.shield.regist_elem = buf[ 0];
		}
	}

	return buf_itemdata;
}

int item_writeparam(const char *ptr_filename, const itemdata_t *ptr_itemdata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_itemdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* ACe̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* ACe̐lnf[^o͂() */
	fprintf(fp, "No.      O               "
				"  i "
				"\ "
				"U d\n");
	for (i = REGION_WEAPON_BEGIN; i <= REGION_WEAPON_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :  %5ld, %3d, %3d, %3d\n", i, i,
				item_getname(i),
				ptr_itemdata[i].price,
				ptr_itemdata[i].power,
				ptr_itemdata[i].param.weapon.attack_power,
				ptr_itemdata[i].param.weapon.weight);
	}

	/* ACe̐lnf[^o͂(h) */
	fprintf(fp, "No.      O               "
				"  i "
				"\ "
				"ha h h h hM h h h "
				"r p  s f ̗ "
				"d\n");
	for (i = REGION_PROTECTOR_BEGIN; i <= REGION_PROTECTOR_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :  %5ld, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\n", i, i,
				item_getname(i),
				ptr_itemdata[i].price,
				ptr_itemdata[i].power,
				ptr_itemdata[i].param.protector.defense_slash,
				ptr_itemdata[i].param.protector.defense_knock,
				ptr_itemdata[i].param.protector.defense_poke,
				ptr_itemdata[i].param.protector.defense_shoot,
				ptr_itemdata[i].param.protector.defense_fire,
				ptr_itemdata[i].param.protector.defense_ice,
				ptr_itemdata[i].param.protector.defense_thunder,
				ptr_itemdata[i].param.protector.defense_state,
				ptr_itemdata[i].param.protector.increase_strength,
				ptr_itemdata[i].param.protector.increase_dexterity,
				ptr_itemdata[i].param.protector.increase_magic_power,
				ptr_itemdata[i].param.protector.increase_unknown,
				ptr_itemdata[i].param.protector.increase_speed,
				ptr_itemdata[i].param.protector.increase_stamina,
				ptr_itemdata[i].param.protector.weight);
	}

	/* ACe̐lnf[^o͂() */
	fprintf(fp, "No.      O               "
				"  i "
				"\ "
				"h  d\n");
	for (i = REGION_SHIELD_BEGIN; i <= REGION_SHIELD_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :  %5ld, %3d, %3d, %3d, %3d\n", i, i,
				item_getname(i),
				ptr_itemdata[i].price,
				ptr_itemdata[i].power,
				ptr_itemdata[i].param.shield.defense_power,
				ptr_itemdata[i].param.shield.evade_rate,
				ptr_itemdata[i].param.shield.weight);
	}

	/* ACe̐lnf[^o͂(̑) */
	fprintf(fp, "No.      O               "
				"  i\n");
	for (i = REGION_OTHER_BEGIN; i <= REGION_OTHER_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :  %5ld\n", i, i,
				item_getname(i),
				ptr_itemdata[i].price);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int item_writeregist(const char *ptr_filename, const itemdata_t *ptr_itemdata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_itemdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* ACȇϐnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* ACȇϐnf[^o͂() */
	fprintf(fp, "No.      O\n");
	for (i = REGION_WEAPON_BEGIN; i <= REGION_WEAPON_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :\n", i, i,
				item_getname(i));
	}

	/* ACȇϐnf[^o͂(h) */
	fprintf(fp, "No.      O               "
				"ԕωϐ\n");
	for (i = REGION_PROTECTOR_BEGIN; i <= REGION_PROTECTOR_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s : ", i, i,
				item_getname(i));
		str_printstate(fp, ptr_itemdata[i].param.protector.regist_state, STATE_ALL);
		fputc('\n', fp);
	}

	/* ACȇϐnf[^o͂() */
	fprintf(fp, "No.      O               "
				"h䑮\n");
	for (i = REGION_SHIELD_BEGIN; i <= REGION_SHIELD_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s : ", i, i,
				item_getname(i));
		str_printelem(fp, ptr_itemdata[i].param.shield.regist_elem, ELEM_ALL);
		fputc('\n', fp);
	}

	/* ACȇϐnf[^o͂(̑) */
	fprintf(fp, "No.      O\n");
	for (i = REGION_OTHER_BEGIN; i <= REGION_OTHER_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :\n", i, i,
				item_getname(i));
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int item_writeeffect(const char *ptr_filename, const itemdata_t *ptr_itemdata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "--------------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_itemdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* ACěʃf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* ACěʃf[^o͂() */
	fprintf(fp, "No.      O               "
				"ŗLZ1           ŗLZ2\n");
	for (i = REGION_WEAPON_BEGIN; i <= REGION_WEAPON_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s : ", i, i,
				item_getname(i));
		for (j = 0; j < 2; j++)
		{
			if (j != 0)
				fprintf(fp, ", ");
			fprintf(fp, "%-20s",
					(ptr_itemdata[i].param.weapon.tech_no[j] != 0xff) ?
					tech_getname(ptr_itemdata[i].param.weapon.tech_no[j]) : str_nothing);
		}
		fputc('\n', fp);
	}

	/* ACěʃf[^o͂(h) */
	fprintf(fp, "No.      O\n");
	for (i = REGION_PROTECTOR_BEGIN; i <= REGION_PROTECTOR_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :\n", i, i,
				item_getname(i));
	}

	/* ACěʃf[^o͂() */
	fprintf(fp, "No.      O\n");
	for (i = REGION_SHIELD_BEGIN; i <= REGION_SHIELD_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :\n", i, i,
				item_getname(i));
	}

	/* ACěʃf[^o͂(̑) */
	fprintf(fp, "No.      O\n");
	for (i = REGION_OTHER_BEGIN; i <= REGION_OTHER_END; i++)
	{
		fprintf(fp, "%03d(%02X): %-16s :\n", i, i,
				item_getname(i));
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
