/*
 *	rs2monster.c
 *		Copyright (C)2002-2004 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "rs2decode.h"
#include "rs2item.h"
#include "rs2tech.h"
#include "rs2monster.h"

#define FILEPOS_MONDATA		0x1e7700
#define FILEPOS_MONNAME		0x1dcc80
#define FILEPOS_MONTP		0x1e6f62
#define FILEPOS_MONACT		0x1e9f00

#define NUM_MON				320
#define NUM_MONACT			256
#define SIZEOF_MONDATA		32
#define SIZEOF_MONNAME		8
#define SIZEOF_MONTP		2
#define SIZEOF_MONACT		8

static void mon_setname(void);

static char tbl_monname[NUM_MON][256];	/* X^[̖Oi[e[u */
static int flag_setmonname = 0;			/* tbl_monnameɃf[^i[Ă邩ǂ */

const char *mon_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_MON <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setmonname)
		mon_setname();

	return tbl_monname[index];
}

static void mon_setname(void)
{
	int i;

	/* X^[̖Of[^ */
	for (i = 0; i < NUM_MON; i++)
		*tbl_monname[i] = '\0';

	/* X^[̖Of[^ǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONNAME];

		file_read_sfc(buf, FILEPOS_MONNAME + SIZEOF_MONNAME * i, SIZEOF_MONNAME);
		dec_string(tbl_monname[i], buf, sizeof(tbl_monname[i]), SIZEOF_MONNAME);
	}

	flag_setmonname = 1;
}

mondata_t *mon_readdata(void)
{
	int i, j, k;
	mondata_t *buf_mondata = NULL;
	int tbl_action[NUM_MONACT][SIZEOF_MONACT];

	/* X^[̊ef[^i[obt@mۂ */
	if ((buf_mondata = malloc(sizeof(mondata_t) * NUM_MON)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[̍sp^[ǂݍ */
	for (i = 0; i < NUM_MONACT; i++)
	{
		unsigned char buf[SIZEOF_MONACT];

		file_read_sfc(buf, FILEPOS_MONACT + SIZEOF_MONACT * i, SIZEOF_MONACT);
		for (j = 0; j < SIZEOF_MONACT; j++)
			tbl_action[i][j] = buf[j];
	}

	/* X^[̋Zp_ǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONTP];

		file_read_sfc(buf, FILEPOS_MONTP + SIZEOF_MONTP * i, SIZEOF_MONTP);
		buf_mondata[i].technical_point = (buf[1] << 8) | buf[0];
	}

	/* X^[̊ef[^ǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONDATA];

		file_read_sfc(buf, FILEPOS_MONDATA + SIZEOF_MONDATA * i, SIZEOF_MONDATA);

		/* lnf[^ */
		buf_mondata[i].hit_point            = (buf[ 2] << 8) | buf[ 1];
		buf_mondata[i].life_point           = buf[11];
		buf_mondata[i].level_tech           = buf[ 3];
		buf_mondata[i].level_mag            = buf[ 4];
		buf_mondata[i].strength             = buf[ 5];
		buf_mondata[i].dexterity            = buf[ 6];
		buf_mondata[i].magic_power          = buf[ 7];
		buf_mondata[i].unknown              = buf[ 8];
		buf_mondata[i].speed                = buf[ 9];
		buf_mondata[i].stamina              = buf[10];
		buf_mondata[i].defense_slash        = buf[12];
		buf_mondata[i].defense_knock        = buf[13];
		buf_mondata[i].defense_poke         = buf[14];
		buf_mondata[i].defense_shoot        = buf[15];
		buf_mondata[i].defense_fire         = buf[16];
		buf_mondata[i].defense_ice          = buf[17];
		buf_mondata[i].defense_thunder      = buf[18];
		buf_mondata[i].defense_state        = buf[19];
		buf_mondata[i].shield_defense_power = buf[22];
		buf_mondata[i].shield_evade_rate    = buf[21] & 0x7f;
		buf_mondata[i].type_race            = buf[ 0] & 0x0f;
		buf_mondata[i].type_act_select      = buf[28] & 0x03;
		buf_mondata[i].type_act             = buf[31];

		/* tOnf[^ */
		buf_mondata[i].shield_regist_elem = buf[20];
		buf_mondata[i].regist_state       = buf[23];

		buf_mondata[i].flag_woman  = ((buf[ 0] & 0x80) != 0x00);
		buf_mondata[i].flag_undead = ((buf[ 0] & 0x40) != 0x00);
		buf_mondata[i].flag_boss   = ((buf[ 0] & 0x20) != 0x00);
		buf_mondata[i].flag_frying = ((buf[ 0] & 0x10) != 0x00);

		buf_mondata[i].flag_target_emperor = ((buf[28] & 0x80) != 0x00);
		buf_mondata[i].flag_target_male    = ((buf[28] & 0x40) != 0x00);
		buf_mondata[i].flag_target_female  = ((buf[28] & 0x20) != 0x00);
		buf_mondata[i].flag_target_hp_low  = ((buf[28] & 0x10) != 0x00);
		buf_mondata[i].flag_target_hp_high = ((buf[28] & 0x08) != 0x00);
		buf_mondata[i].flag_target_nouse   = ((buf[28] & 0x04) != 0x00);

		/* f[^ */
		for (j = 0; j < 2; j++)
			buf_mondata[i].weapon_no[j] = buf[j + 24];

		/* ACef[^ */
		for (j = 0; j < 2; j++)
			buf_mondata[i].item_no[j] = buf[j + 29];

		/* sp^[f[^ */
		for (j = 0; j < 2; j++)
		{
			for (k = 0; k < SIZEOF_MONACT; k++)
				buf_mondata[i].action_no[j][k] = tbl_action[buf[j + 26]][k];
		}

		/* Z|CgEp|Cg̎Zo */
		buf_mondata[i].tech_point = buf_mondata[i].level_tech * 7;		/* 5nׂĂ̋Zxl */
		if (buf_mondata[i].tech_point > 250)
			buf_mondata[i].tech_point = 250;
		buf_mondata[i].mag_point  = buf_mondata[i].level_mag * 8 + 5;	/* 6nׂĂ̏pxl */
		if (buf_mondata[i].mag_point > 250)
			buf_mondata[i].mag_point = 250;
	}

	return buf_mondata;
}

int mon_writeparam(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̐lnf[^o͂ */
	fprintf(fp, "No.       O               "
				"Zp_     HP   LP   WP   JP "
				"r p  s f ̗ "
				"ZLV pLV "
				"ha h h h hM h h h "
				"h \n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%03X): %-16s :  %5ld, %5ld, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\n", i, i,
				mon_getname(i),
				ptr_mondata[i].technical_point,
				ptr_mondata[i].hit_point,
				ptr_mondata[i].life_point,
				ptr_mondata[i].tech_point,
				ptr_mondata[i].mag_point,
				ptr_mondata[i].strength,
				ptr_mondata[i].dexterity,
				ptr_mondata[i].magic_power,
				ptr_mondata[i].unknown,
				ptr_mondata[i].speed,
				ptr_mondata[i].stamina,
				ptr_mondata[i].level_tech,
				ptr_mondata[i].level_mag,
				ptr_mondata[i].defense_slash,
				ptr_mondata[i].defense_knock,
				ptr_mondata[i].defense_poke,
				ptr_mondata[i].defense_shoot,
				ptr_mondata[i].defense_fire,
				ptr_mondata[i].defense_ice,
				ptr_mondata[i].defense_thunder,
				ptr_mondata[i].defense_state,
				ptr_mondata[i].shield_defense_power,
				ptr_mondata[i].shield_evade_rate);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeregist(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̑ϐnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̑ϐnf[^o͂ */
	fprintf(fp, "No.       O               "
				"̖h䑮      "
				"ԕωϐ\n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%03X): %-16s : ", i, i, mon_getname(i));

		/* ̖h䑮 */
		str_printelem(fp, ptr_mondata[i].shield_regist_elem, ELEM_ALL);
		fprintf(fp, "  ");

		/* ԕωւ̑ϐ */
		str_printstate(fp, ptr_mondata[i].regist_state, STATE_ALL);
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeflag(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̃tOnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̃tOnf[^o͂ */
	fprintf(fp, "No.       O               "
				" s {X s 푰\n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%03X): %-16s :  %2s   %2s   %2s   %2s  %s\n", i, i,
				mon_getname(i),
				str_getflag(ptr_mondata[i].flag_woman),
				str_getflag(ptr_mondata[i].flag_undead),
				str_getflag(ptr_mondata[i].flag_boss),
				str_getflag(ptr_mondata[i].flag_frying),
				str_getrace(ptr_mondata[i].type_race));
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeitem(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "----------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̏ACef[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̏ACef[^o͂ */
	fprintf(fp, "No.       O               "
				"m            "
				"m\n");
	for (i = 0; i < NUM_MON; i++)
	{
		const char *ptr_name[2];

		for (j = 0; j < 2; j++)
			ptr_name[j] = (ptr_mondata[i].item_no[j] != 0xff) ? item_getname(ptr_mondata[i].item_no[j]) : str_nothing;

		fprintf(fp, "%03d(%03X): %-16s : ", i, i, mon_getname(i));
		for (j = 0; j < 2; j++)
		{
			if (j != 0)
				fprintf(fp, ", ");
			fprintf(fp, "%-16s", ptr_name[j]);
		}
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeequip(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "----------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̑ACef[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̑ACef[^o͂ */
	fprintf(fp, "No.       O               "
				"1             "
				"2\n");
	for (i = 0; i < NUM_MON; i++)
	{
		const char *ptr_name[2];

		for (j = 0; j < 2; j++)
			ptr_name[j] = (ptr_mondata[i].weapon_no[j] != 0xfe) ? item_getname(ptr_mondata[i].weapon_no[j]) : str_nothing;

		fprintf(fp, "%03d(%03X): %-16s : ", i, i, mon_getname(i));
		for (j = 0; j < 2; j++)
		{
			if (j != 0)
				fprintf(fp, ", ");
			fprintf(fp, "%-16s", ptr_name[j]);
		}
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeact(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j, k;
	FILE *fp = NULL;

	const int tbl_act_select[][2] =
	{
		{7, 1},
		{5, 3},
		{2, 6},
		{1, 7},
	};

	assert(sizeof(tbl_act_select) / sizeof(tbl_act_select[0]) == 4);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̍sp^[f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̍sp^[f[^o͂ */
	for (i = 0; i < NUM_MON; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%03d(%02X): %s]\n", i, i, mon_getname(i));

		fprintf(fp, "UڕW:");
		if (ptr_mondata[i].flag_target_emperor)
			fprintf(fp, " c");
/*
		if (ptr_mondata[i].flag_target_male)
			fprintf(fp, " jLN^");
		if (ptr_mondata[i].flag_target_female)
			fprintf(fp, " LN^");
*/
		if (ptr_mondata[i].flag_target_hp_low)
			fprintf(fp, " HPLN^");
		if (ptr_mondata[i].flag_target_hp_high)
			fprintf(fp, " HPLN^");
/*
		if (ptr_mondata[i].flag_target_nouse)
			fprintf(fp, " gp");
*/
		fputc('\n', fp);

		assert(0 <= ptr_mondata[i].type_act_select && ptr_mondata[i].type_act_select < 4);

		fprintf(fp, "sp^[1I: %d/8\n", tbl_act_select[ptr_mondata[i].type_act_select][0]);
		fprintf(fp, "sp^[2I: %d/8\n", tbl_act_select[ptr_mondata[i].type_act_select][1]);
/*		fprintf(fp, "s^Cv: %3d\n", ptr_mondata[i].type_act);	*/

		fprintf(fp, "No.   s\n");
		for (j = 0; j < 2; j++)
		{
			for (k = 0; k < 8; k++)
				fprintf(fp, "%1d-%02d: %s\n", j + 1, k + 1, tech_getname(ptr_mondata[i].action_no[j][k]));
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
