/*
 *	rs3character.c
 *		Copyright (C)2002 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "rs3decode.h"
#include "rs3define.h"
#include "rs3character.h"

#define FILEPOS_CHRDATA			0x3e0600
#define FILEPOS_CHRLVUP			0x3e31bf
#define FILEPOS_CHRNAME			0x3d1480
#define NUM_CHR					48
#define NUM_CHR_NAME			256
#define SIZEOF_CHRNAME			8
#define SIZEOF_CHRDATA			48
#define SIZEOF_CHRLVUP			16

static void chr_setname(void);

static char tbl_chrname[NUM_CHR_NAME][256];	/* LN^̖Oi[e[u */
static int flag_setchrname = 0;				/* tbl_chrnameɃf[^i[Ă邩ǂ */

const char *chr_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_CHR <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setchrname)
		chr_setname();

	if (0 <= index && index < 41)
		return tbl_chrname[index];
	else if (41 <= index && index < 47)
		return tbl_chrname[index - 41];
	else
		return tbl_chrname[16];
}

const char *chr_getname2(int index)
{
	/* `FbN */
	if (index < 0 || NUM_CHR_NAME <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setchrname)
		chr_setname();

	return tbl_chrname[index];
}

static void chr_setname(void)
{
	int i;

	/* LN^̖Of[^ */
	for (i = 0; i < NUM_CHR_NAME; i++)
		*tbl_chrname[i] = '\0';

	/* LN^̖Of[^ǂݍ */
	for (i = 0; i < NUM_CHR_NAME; i++)
	{
		unsigned char buf[SIZEOF_CHRNAME];

		file_read_sfc(buf, FILEPOS_CHRNAME + SIZEOF_CHRNAME * i, SIZEOF_CHRNAME);
		dec_string(tbl_chrname[i], buf, sizeof(tbl_chrname[i]), SIZEOF_CHRNAME);
	}

	flag_setchrname = 1;
}

chrdata_t *chr_readdata(void)
{
	int i, j;
	chrdata_t *buf_chrdata = NULL;

	/* LN^̊ef[^i[obt@mۂ */
	if ((buf_chrdata = malloc(sizeof(chrdata_t) * NUM_CHR)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* LN^̊ef[^ǂݍ */
	for (i = 0; i < NUM_CHR; i++)
	{
		unsigned char buf[SIZEOF_CHRDATA];

		file_read_sfc(buf, FILEPOS_CHRDATA + SIZEOF_CHRDATA * i, SIZEOF_CHRDATA);

		/* lnf[^ */
		buf_chrdata[i].hit_point       = ((buf[ 4] & 0x7f) << 8) | buf[ 3];
		buf_chrdata[i].life_point      = buf[ 5] & 0x7f;
		buf_chrdata[i].strength        = buf[ 6];
		buf_chrdata[i].dexterity       = buf[ 7];
		buf_chrdata[i].speed           = buf[ 8];
		buf_chrdata[i].stamina         = buf[ 9];
		buf_chrdata[i].magic_power     = buf[10];
		buf_chrdata[i].will_power      = buf[11];
		buf_chrdata[i].charm           = buf[12];
		buf_chrdata[i].level_sword     = buf[13] & 0x7f;
		buf_chrdata[i].level_axe       = buf[14] & 0x7f;
		buf_chrdata[i].level_spear     = buf[15] & 0x7f;
		buf_chrdata[i].level_bow       = buf[16] & 0x7f;
		buf_chrdata[i].level_punch     = buf[17] & 0x7f;
		buf_chrdata[i].level_wind      = (((buf[18] >> 5) & 0x03) == 0x00) ? (buf[18] & 0x1f) : 0;
		buf_chrdata[i].level_fire      = (((buf[18] >> 5) & 0x03) == 0x01) ? (buf[18] & 0x1f) : 0;
		buf_chrdata[i].level_earth     = (((buf[18] >> 5) & 0x03) == 0x02) ? (buf[18] & 0x1f) : 0;
		buf_chrdata[i].level_water     = (((buf[18] >> 5) & 0x03) == 0x03) ? (buf[18] & 0x1f) : 0;
		buf_chrdata[i].level_sun       = (((buf[19] >> 5) & 0x03) == 0x00) ? (buf[19] & 0x1f) : 0;
		buf_chrdata[i].level_moon      = (((buf[19] >> 5) & 0x03) == 0x01) ? (buf[19] & 0x1f) : 0;
		buf_chrdata[i].level_unknown   = buf[20] & 0x7f;
		buf_chrdata[i].type_star       = buf[40];
		buf_chrdata[i].type_weapon     = buf[42];
		buf_chrdata[i].type_mag        = buf[41];
		buf_chrdata[i].type_tech_learn = buf[22];
		buf_chrdata[i].type_levelup    = buf[21];
		buf_chrdata[i].formation_no    = buf[39];

		/* tOnf[^ */
		buf_chrdata[i].flag_man     = ((buf[ 2] & 0x80) != 0x00);
		buf_chrdata[i].flag_woman   = ((buf[ 2] & 0x40) != 0x00);
		buf_chrdata[i].flag_undead  = ((buf[ 2] & 0x20) != 0x00);
		buf_chrdata[i].flag_unknown = ((buf[ 2] & 0x10) != 0x00);

		buf_chrdata[i].flag_grow_hit_point     = ((buf[ 4] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_sword   = ((buf[13] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_axe     = ((buf[14] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_spear   = ((buf[15] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_bow     = ((buf[16] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_punch   = ((buf[17] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_earth   = ((buf[18] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_heaven  = ((buf[19] & 0x80) != 0x00);
		buf_chrdata[i].flag_grow_level_unknown = ((buf[20] & 0x80) != 0x00);

		/* CĂZ */
		for (j = 0; j < 4; j++)
			buf_chrdata[i].tech_no[j] = buf[j + 23];

		/* CĂp */
		for (j = 0; j < 4; j++)
			buf_chrdata[i].mag_no[j] = buf[j + 27];

		/* ACe */
		for (j = 0; j < 8; j++)
			buf_chrdata[i].equipment_no[j] = buf[j + 31];
	}

	/* LN^̐f[^ǂݍ */
	for (i = 0; i < NUM_CHR; i++)
	{
		unsigned char buf[SIZEOF_CHRLVUP];

		file_read_sfc(buf, FILEPOS_CHRLVUP + SIZEOF_CHRLVUP * i, SIZEOF_CHRLVUP);

		buf_chrdata[i].lvup_hit_point  = buf[15];
		buf_chrdata[i].lvup_tech_point = buf[13];
		buf_chrdata[i].lvup_mag_point  = buf[14];
		buf_chrdata[i].lvup_sword      = buf[ 0];
		buf_chrdata[i].lvup_axe        = buf[ 1];
		buf_chrdata[i].lvup_spear      = buf[ 2];
		buf_chrdata[i].lvup_bow        = buf[ 3];
		buf_chrdata[i].lvup_punch      = buf[ 4];
		buf_chrdata[i].lvup_wind       = buf[ 5];
		buf_chrdata[i].lvup_fire       = buf[ 6];
		buf_chrdata[i].lvup_earth      = buf[ 7];
		buf_chrdata[i].lvup_water      = buf[ 8];
		buf_chrdata[i].lvup_sun        = buf[ 9];
		buf_chrdata[i].lvup_moon       = buf[10];
	}

	return buf_chrdata;
}

int chr_writeparam(const char *ptr_filename, const chrdata_t *ptr_chrdata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_chrdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* LN^̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* LN^̐lnf[^o͂ */
	fprintf(fp, "No.       O               "
				"    HP "
				"r p f ̗  ӎu  "
				"           | ̏p "
				" 钹   z    "
				"s\n");
	for (i = 0; i < NUM_CHR; i++)
	{
		fprintf(fp, "%03d(%03X): %-16s :  %5ld, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\n", i, i,
				chr_getname(i),
				ptr_chrdata[i].hit_point,
				ptr_chrdata[i].strength,
				ptr_chrdata[i].dexterity,
				ptr_chrdata[i].speed,
				ptr_chrdata[i].stamina,
				ptr_chrdata[i].magic_power,
				ptr_chrdata[i].will_power,
				ptr_chrdata[i].charm,
				ptr_chrdata[i].level_sword,
				ptr_chrdata[i].level_axe,
				ptr_chrdata[i].level_spear,
				ptr_chrdata[i].level_bow,
				ptr_chrdata[i].level_punch,
				ptr_chrdata[i].level_wind,
				ptr_chrdata[i].level_fire,
				ptr_chrdata[i].level_earth,
				ptr_chrdata[i].level_water,
				ptr_chrdata[i].level_sun,
				ptr_chrdata[i].level_moon,
				ptr_chrdata[i].level_unknown);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int chr_writelvup(const char *ptr_filename, const chrdata_t *ptr_chrdata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_chrdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* LN^̐f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* LN^̐f[^o͂ */
	fprintf(fp, "No.       O               "
				"  HP   WP   JP "
				"           | ̏p "
				" 钹   z    "
				"   TYPE\n");
	for (i = 0; i < NUM_CHR; i++)
	{
		fprintf(fp, "%03d(%03X): %-16s :  %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, TYPE%02d\n", i, i,
				chr_getname(i),
				ptr_chrdata[i].lvup_hit_point,
				ptr_chrdata[i].lvup_tech_point,
				ptr_chrdata[i].lvup_mag_point,
				ptr_chrdata[i].lvup_sword,
				ptr_chrdata[i].lvup_axe,
				ptr_chrdata[i].lvup_spear,
				ptr_chrdata[i].lvup_bow,
				ptr_chrdata[i].lvup_punch,
				ptr_chrdata[i].lvup_wind,
				ptr_chrdata[i].lvup_fire,
				ptr_chrdata[i].lvup_earth,
				ptr_chrdata[i].lvup_water,
				ptr_chrdata[i].lvup_sun,
				ptr_chrdata[i].lvup_moon,
				ptr_chrdata[i].type_levelup);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int chr_writeflag(const char *ptr_filename, const chrdata_t *ptr_chrdata)
{
	int i;
	FILE *fp = NULL;
	const char *str_star[] =
	{
		"ΐ",
		"壘f",
		"",
		"",
		"C",
	};
	const char *str_weapon[] =
	{
		"",
		"匕",
		"",
		"_",
		"",
		"",
		"|",
		"̏p",
		"Ȃ",
	};
	const char *str_mag[] =
	{
		"",
		"钹",
		"z",
		"",
		"",
	};

	assert(sizeof(str_star) / sizeof(str_star[0]) == 5);
	assert(sizeof(str_weapon) / sizeof(str_weapon[0]) == 9);
	assert(sizeof(str_mag) / sizeof(str_mag[0]) == 5);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_chrdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* LN^̃tOnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* LN^̃tOnf[^o͂ */
	fprintf(fp, "No.       O               "
				"j  s s "
				" HP            |  ̏p np Vp s  "
				"h    p    MTYPE\n");
	for (i = 0; i < NUM_CHR; i++)
	{
		if (ptr_chrdata[i].type_star >= 5)
			break;
		if (ptr_chrdata[i].type_weapon >= 9)
			break;
		if (ptr_chrdata[i].type_mag >= 5)
			break;

		fprintf(fp, "%03d(%03X): %-16s :  %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s   %-4s  %-4s  %-4s  TYPE%02d\n", i, i,
				chr_getname(i),
				str_getflag(ptr_chrdata[i].flag_man),
				str_getflag(ptr_chrdata[i].flag_woman),
				str_getflag(ptr_chrdata[i].flag_undead),
				str_getflag(ptr_chrdata[i].flag_unknown),
				str_getflag(ptr_chrdata[i].flag_grow_hit_point),
				str_getflag(ptr_chrdata[i].flag_grow_level_sword),
				str_getflag(ptr_chrdata[i].flag_grow_level_axe),
				str_getflag(ptr_chrdata[i].flag_grow_level_spear),
				str_getflag(ptr_chrdata[i].flag_grow_level_bow),
				str_getflag(ptr_chrdata[i].flag_grow_level_punch),
				str_getflag(ptr_chrdata[i].flag_grow_level_earth),
				str_getflag(ptr_chrdata[i].flag_grow_level_heaven),
				str_getflag(ptr_chrdata[i].flag_grow_level_unknown),
				str_star[ptr_chrdata[i].type_star],
				str_weapon[ptr_chrdata[i].type_weapon],
				str_mag[ptr_chrdata[i].type_mag],
				ptr_chrdata[i].type_tech_learn);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
