/*
 *	rs3define.c
 *		Copyright (C)2002, 2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "rs3define.h"

#define elem_maskbit(elem_no)	(1U << (elem_no))
#define state_maskbit(state_no)	(1U << (state_no))

#define STR_NOTHING		"E"
#define STR_FLAG_TRUE	""
#define STR_FLAG_FALSE	"~"

static const char *str_element[/* NUM_ELEMENT */] =
{
	"",
	"",
	"",
	"M",
	"",
	"",
	"",
	"a",
};

static const char *str_state[/* NUM_STATE */] =
{
	"X",	/* X^ */
	"",	/* ғ */
	"",	/* _ */
	"",	/* È */
	"",	/*  */
	"",	/*  */
	"??",	/* s */
	"",	/*  */
};

const char *str_getflag(int flag)
{
	return (flag != 0x00) ? STR_FLAG_TRUE : STR_FLAG_FALSE;
}

const char *str_getelem(elem_t elem, int index)
{
	assert(sizeof(str_element) / sizeof(str_element[0]) == NUM_ELEMENT);

	/* `FbN */
	if (index < 0 || NUM_ELEMENT <= index)
	{
		assert(0);
		return "";
	}

	return ((elem & elem_maskbit(index)) != 0x00) ? str_element[index] : "";
}

void str_printelem(FILE *stream, elem_t elem, elem_t elem_mask)
{
	int i;

	assert(sizeof(str_element) / sizeof(str_element[0]) == NUM_ELEMENT);

	/* `FbN */
	if (stream == NULL)
	{
		assert(0);
		return;
	}

	for (i = 0; i < NUM_ELEMENT; i++)
	{
		if ((elem_mask & elem_maskbit(7 - i)) != 0x00)
			fprintf(stream, "%-2s", ((elem & elem_maskbit(7 - i)) != 0x00) ? str_element[7 - i] : STR_NOTHING);
	}
}

const char *str_getstate(state_t state, int index)
{
	assert(sizeof(str_state) / sizeof(str_state[0]) == NUM_STATE);

	/* `FbN */
	if (index < 0 || NUM_STATE <= index)
	{
		assert(0);
		return "";
	}

	return ((state & state_maskbit(index)) != 0x00) ? str_state[index] : "";
}

void str_printstate(FILE *stream, state_t state, state_t state_mask)
{
	int i;

	assert(sizeof(str_state) / sizeof(str_state[0]) == NUM_STATE);

	/* `FbN */
	if (stream == NULL)
	{
		assert(0);
		return;
	}

	for (i = 0; i < NUM_STATE; i++)
	{
		if ((state_mask & state_maskbit(7 - i)) != 0x00)
			fprintf(stream, "%-2s", ((state & state_maskbit(7 - i)) != 0x00) ? str_state[7 - i] : STR_NOTHING);
	}
}

const char *str_getrace(int index)
{
	static const char *str_racetype[] =
	{
		"----",
		"bl",
		"b",
		"",
		"",
		"",
		"",
		"",
		"A",
		"@",
		"",
		"",
		"s",
		"",
		"[",
		"l",
	};

	assert(sizeof(str_racetype) / sizeof(str_racetype[0]) == 16);

	/* `FbN */
	if (index < 0 || 16 <= index)
	{
		assert(0);
		return "";
	}

	return str_racetype[index];
}

const char *str_gethealing(int index)
{
	static const char *str_healingtype[] =
	{
		"------",
		"",
		"钹",
		"",
		"",
		"z",
		"",
		"ArX",
	};

	assert(sizeof(str_healingtype) / sizeof(str_healingtype[0]) == 8);

	/* `FbN */
	if (index < 0 || 8 <= index)
	{
		assert(0);
		return "";
	}

	return str_healingtype[index];
}

int item_getdefense(int defense, int regist)
{
	int ret = 0;

	/* `FbN */
	if (regist < 0 || 8 <= regist || defense < 0)
	{
		assert(0);
		return 0;
	}

	switch (regist)
	{
	case 0:		/* ϐȂ */
	case 6:		/* ϐ"(~)" */
		ret = defense;
		break;
	case 1:		/* ϐ"~" */
		ret = 0;
		break;
	case 2:		/* ϐ"" */
		ret = defense / 2;
		break;
	case 3:		/* ϐ"" */
		ret = defense + defense / 2;
		break;
	case 4:		/* ϐ"+" */
		ret = defense * 2;
		break;
	case 5:		/* ϐ"" */
		ret = defense * 3;
		break;
	case 7:		/* ϐ"" */
		ret = 128;
		break;
	default:
		assert(0);
	};

	return ret;
}
