/*
 *	rs3monster.c
 *		Copyright (C)2002, 2003 Shingo Endo <s-endo@yk.rim.or.jp>
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "errmes.h"
#include "filesfc.h"
#include "rs3decode.h"
#include "rs3item.h"
#include "rs3tech.h"
#include "rs3monster.h"

#define FILEPOS_MONDATA		0x3e3f8b
#define FILEPOS_MONNAME		0x3d0a80
#define FILEPOS_MONITEM		0x3e678b
#define FILEPOS_MONACT		0x3e5ecb
#define FILEPOS_MONPARTY	0x3e756b
#define FILEPOS_MONENCMAIN	0x3e77bb
#define FILEPOS_MONENCSUB	0x3e79bb
#define FILEPOS_MONSIZE		0x26e400
#define NUM_MON				320
#define NUM_MONITEM			64
#define NUM_MONACT			256
#define NUM_MONPARTY		16
#define NUM_MONENCMAIN		16
#define NUM_MONENCSUB		15
#define SIZEOF_MONDATA		25
#define SIZEOF_MONNAME		8
#define SIZEOF_MONITEM		5
#define SIZEOF_MONACT		8
#define SIZEOF_MONPARTY		13
#define SIZEOF_MONENCMAIN	(2 * 16)
#define SIZEOF_MONENCSUB	(2 * 48)

static void mon_setname(void);

static char tbl_monname[NUM_MON][256];	/* X^[̖Oi[e[u */
static int flag_setmonname = 0;			/* tbl_monnameɃf[^i[Ă邩ǂ */

const char *mon_getname(int index)
{
	/* `FbN */
	if (index < 0 || NUM_MON <= index)
	{
		assert(0);
		return "";
	}

	/* e[uɒlݒ肳ĂȂꍇ́Alݒ肷 */
	if (!flag_setmonname)
		mon_setname();

	return tbl_monname[index];
}

static void mon_setname(void)
{
	int i;

	/* X^[̖Of[^ */
	for (i = 0; i < NUM_MON; i++)
		*tbl_monname[i] = '\0';

	/* X^[̖Of[^ǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONNAME];

		file_read_sfc(buf, FILEPOS_MONNAME + SIZEOF_MONNAME * i, SIZEOF_MONNAME);
		dec_string(tbl_monname[i], buf, sizeof(tbl_monname[i]), SIZEOF_MONNAME);
	}

	flag_setmonname = 1;
}

mondata_t *mon_readdata(void)
{
	int i, j, k;
	mondata_t *buf_mondata = NULL;
	int tbl_item[NUM_MONITEM][SIZEOF_MONITEM];
	int tbl_action[NUM_MONACT][SIZEOF_MONACT];

	/* X^[̊ef[^i[obt@mۂ */
	if ((buf_mondata = malloc(sizeof(mondata_t) * NUM_MON)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[̏ACef[^ǂݍ */
	for (i = 0; i < NUM_MONITEM; i++)
	{
		unsigned char buf[SIZEOF_MONITEM];

		file_read_sfc(buf, FILEPOS_MONITEM + SIZEOF_MONITEM * i, SIZEOF_MONITEM);
		for (j = 0; j < SIZEOF_MONITEM; j++)
			tbl_item[i][j] = buf[j];
	}

	/* X^[̍sp^[ǂݍ */
	for (i = 0; i < NUM_MONACT; i++)
	{
		unsigned char buf[SIZEOF_MONACT];

		file_read_sfc(buf, FILEPOS_MONACT + SIZEOF_MONACT * i, SIZEOF_MONACT);
		for (j = 0; j < SIZEOF_MONACT; j++)
			tbl_action[i][j] = buf[j];
	}

	/* X^[̊ef[^ǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char buf[SIZEOF_MONDATA];

		file_read_sfc(buf, FILEPOS_MONDATA + SIZEOF_MONDATA * i, SIZEOF_MONDATA);

		/* lnf[^ */
		buf_mondata[i].hit_point        = (buf[ 5] << 8) | buf[ 4];
		buf_mondata[i].strength         = buf[ 6];
		buf_mondata[i].dexterity        = buf[ 7];
		buf_mondata[i].speed            = buf[ 8];
		buf_mondata[i].stamina          = buf[ 9];
		buf_mondata[i].magic_power      = buf[10];
		buf_mondata[i].will_power       = buf[11];
		buf_mondata[i].charm            = buf[12];
		buf_mondata[i].level_tech       = buf[13];
		buf_mondata[i].level_mag        = buf[14];
		buf_mondata[i].level_lvup       = buf[15];
		buf_mondata[i].level_tech_learn = buf[16];
		buf_mondata[i].type_race        = buf[ 0] & 0x0f;
		buf_mondata[i].type_healing     = buf[ 1] & 0x07;

		/* tOnf[^ */
		buf_mondata[i].flag_man     = ((buf[ 0] & 0x80) != 0x00);
		buf_mondata[i].flag_woman   = ((buf[ 0] & 0x40) != 0x00);
		buf_mondata[i].flag_undead  = ((buf[ 0] & 0x20) != 0x00);
		buf_mondata[i].flag_unknown = ((buf[ 0] & 0x10) != 0x00);

		buf_mondata[i].flag_regist_water     = ((buf[ 1] & 0x08) != 0x00);
		buf_mondata[i].flag_regist_throw     = ((buf[ 1] & 0x10) != 0x00);
		buf_mondata[i].flag_regist_soundwave = ((buf[ 1] & 0x20) != 0x00);
		buf_mondata[i].flag_regist_gaze      = ((buf[ 1] & 0x40) != 0x00);
		buf_mondata[i].flag_regist_ground    = ((buf[ 1] & 0x80) != 0x00);

		for (j = 0; j < 2; j++)
			buf_mondata[i].flag_catch_weapon[j] = ((buf[24] & (1U << (7 - j))) == 0);

		/* ACef[^ */
		for (j = 0; j < 5; j++)
			buf_mondata[i].equipment_no[j] = buf[j + 19];

		/* ACef[^ */
		for (j = 0; j < 4; j++)
		{
			int item_no = tbl_item[buf[24] & 0x3f][j];

			if (item_no == 0xfd)
				item_no = buf_mondata[i].equipment_no[0];
			else if (item_no == 0xfe)
				item_no = buf_mondata[i].equipment_no[1];

			buf_mondata[i].item_no[j] = item_no;
		}
		buf_mondata[i].credit = tbl_item[buf[24] & 0x3f][4];

		/* sp^[f[^ */
		for (j = 0; j < 2; j++)
		{
			for (k = 0; k < SIZEOF_MONACT; k++)
				buf_mondata[i].action_no[j][k] = tbl_action[buf[j + 17]][k];
		}

		/* Z|CgEp|Cg(vZs) */
		buf_mondata[i].tech_point = 0;
		buf_mondata[i].mag_point  = 0;
	}

	/* X^[̃TCYǂݍ */
	for (i = 0; i < NUM_MON; i++)
	{
		unsigned char byte;

		file_read_sfc(&byte, FILEPOS_MONSIZE + i, 1);

		buf_mondata[i].size.width  = (byte & 0x0f) * 4;
		buf_mondata[i].size.height = ((byte & 0xf0) >> 4) * 4;
	}

	return buf_mondata;
}

monencmain_t *mon_readencmain(void)
{
	int i, j;
	monencmain_t *buf_monencmain = NULL;
	monparty_t tbl_party[NUM_MONPARTY];

	/* X^[oxf[^([_[X^[)i[obt@mۂ */
	if ((buf_monencmain = malloc(sizeof(monencmain_t) * NUM_MONENCMAIN)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[̃p[eBҐf[^ǂݍ */
	for (i = 0; i < NUM_MONPARTY; i++)
	{
		unsigned char buf[SIZEOF_MONPARTY];

		file_read_sfc(buf, FILEPOS_MONPARTY + SIZEOF_MONPARTY * i, SIZEOF_MONPARTY);

		tbl_party[i].max_num         = buf[0];
		tbl_party[i].max_size.height = buf[1];
		tbl_party[i].max_size.width  = buf[2];

		for (j = 0; j < 5; j++)
		{
			tbl_party[i].pos[j].x = buf[j * 2 + 3];
			tbl_party[i].pos[j].y = buf[j * 2 + 4];
		}
	}

	/* X^[oxf[^([_[X^[)ǂݍ */
	for (i = 0; i < NUM_MONENCMAIN; i++)
	{
		unsigned char buf[SIZEOF_MONENCMAIN];

		file_read_sfc(buf, FILEPOS_MONENCMAIN + SIZEOF_MONENCMAIN * i, SIZEOF_MONENCMAIN);

		for (j = 0; j < 16; j++)
		{
			int idx = buf[j * 2 + 1];

			if (idx > NUM_MONPARTY)
				break;

			buf_monencmain[i].level[j].monster_no = buf[j * 2];
			buf_monencmain[i].level[j].party      = tbl_party[idx];
		}
	}

	return buf_monencmain;
}

monencsub_t *mon_readencsub(void)
{
	int i, j;
	monencsub_t *buf_monencsub = NULL;

	/* X^[oxf[^(]X^[)i[obt@mۂ */
	if ((buf_monencsub = malloc(sizeof(monencsub_t) * NUM_MONENCSUB)) == NULL)
		err_printf(STR_FATAL_ERROR_IN_MALLOC);

	/* X^[oxf[^(]X^[)ǂݍ */
	for (i = 0; i < NUM_MONENCSUB; i++)
	{
		unsigned char buf[SIZEOF_MONENCSUB];

		file_read_sfc(buf, FILEPOS_MONENCSUB + SIZEOF_MONENCSUB * i, SIZEOF_MONENCSUB);

		for (j = 0; j < 48; j++)
		{
			buf_monencsub[i].level[j].monster_no    = buf[j * 2 + 1];
			buf_monencsub[i].level[j].monster_level = buf[j * 2]; 
		}
	}

	return buf_monencsub;
}

int mon_writeparam(const char *ptr_filename, const mondata_t *ptr_mondata, const itemdata_t *ptr_itemdata)
{
	int i, j;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL || ptr_itemdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̐lnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̐lnf[^o͂ */
	fprintf(fp, "No.       O               "
				"    HP "
				"r p f ̗  ӎu  "
				"ZLV pLV LV MLV "
				"ha h h h hM h h h "
				"h \n");
	for (i = 0; i < NUM_MON; i++)
	{
		int defense_slash   = 0;
		int defense_knock   = 0;
		int defense_poke    = 0;
		int defense_shoot   = 0;
		int defense_fire    = 0;
		int defense_ice     = 0;
		int defense_thunder = 0;
		int defense_state   = 0;
		int shield_defense_power = 0;
		int shield_evade_rate    = 0;

		/* ih́E𗦂Zo */
		for (j = 0; j < sizeof(ptr_mondata[i].equipment_no) / sizeof(ptr_mondata[i].equipment_no[0]); j++)
		{
			const int index = ptr_mondata[i].equipment_no[j];

			if (index == 0xff)
				continue;

			switch (ptr_itemdata[index].type)
			{
			case ITEM_TYPE_SHIELD:
				shield_defense_power = ptr_itemdata[index].param.shield.defense_power;
				shield_evade_rate    = ptr_itemdata[index].param.shield.evade_rate;
				break;

			case ITEM_TYPE_PROTECTOR:
				{
					const protector_t *ptr_protector = &ptr_itemdata[index].param.protector;

					defense_slash   += item_getdefense(ptr_protector->defense_power, ptr_protector->defense_slash);
					defense_knock   += item_getdefense(ptr_protector->defense_power, ptr_protector->defense_knock);
					defense_poke    += item_getdefense(ptr_protector->defense_power, ptr_protector->defense_poke);
					defense_shoot   += item_getdefense(ptr_protector->defense_power, ptr_protector->defense_shoot);
					defense_fire    += item_getdefense(ptr_protector->magic_defense_power, ptr_protector->defense_fire);
					defense_ice     += item_getdefense(ptr_protector->magic_defense_power, ptr_protector->defense_ice);
					defense_thunder += item_getdefense(ptr_protector->magic_defense_power, ptr_protector->defense_thunder);
					defense_state   += item_getdefense(ptr_protector->magic_defense_power, ptr_protector->defense_state);
				}
				break;

			case ITEM_TYPE_TOOL:
			case ITEM_TYPE_WEAPON:
			case ITEM_TYPE_OTHER:
				break;

			default:
				assert(0);
			}
		}

		/* o͂ */
		fprintf(fp, "%03d(%03X): %-16s :  %5ld, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d, %3d\n", i, i,
				mon_getname(i),
				ptr_mondata[i].hit_point,
				ptr_mondata[i].strength,
				ptr_mondata[i].dexterity,
				ptr_mondata[i].speed,
				ptr_mondata[i].stamina,
				ptr_mondata[i].magic_power,
				ptr_mondata[i].will_power,
				ptr_mondata[i].charm,
				ptr_mondata[i].level_tech,
				ptr_mondata[i].level_mag,
				ptr_mondata[i].level_lvup,
				ptr_mondata[i].level_tech_learn,
				defense_slash,
				defense_knock,
				defense_poke,
				defense_shoot,
				defense_fire,
				defense_ice,
				defense_thunder,
				defense_state,
				shield_defense_power,
				shield_evade_rate);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeregist(const char *ptr_filename, const mondata_t *ptr_mondata, const itemdata_t *ptr_itemdata)
{
	int i, j;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL || ptr_itemdata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̑ϐnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̑ϐnf[^o͂ */
	fprintf(fp, "No.       O               "
				"̖h䑮      "
				"ԕωϐ\n");
	for (i = 0; i < NUM_MON; i++)
	{
		elem_t  shield_regist_elem = 0;
		state_t regist_state       = 0;

		/* ih́E𗦂Zo */
		for (j = 0; j < sizeof(ptr_mondata[i].equipment_no) / sizeof(ptr_mondata[i].equipment_no[0]); j++)
		{
			const int index = ptr_mondata[i].equipment_no[j];

			if (index == 0xff)
				continue;

			switch (ptr_itemdata[index].type)
			{
			case ITEM_TYPE_SHIELD:
				shield_regist_elem |= ptr_itemdata[index].param.shield.regist_elem;
				break;

			case ITEM_TYPE_PROTECTOR:
				regist_state |= ptr_itemdata[index].param.protector.regist_state;
				break;

			case ITEM_TYPE_TOOL:
			case ITEM_TYPE_WEAPON:
			case ITEM_TYPE_OTHER:
				break;

			default:
				assert(0);
			}
		}
		/* o͂ */
		fprintf(fp, "%03d(%03X): %-16s : ", i, i, mon_getname(i));

		/* ̖h䑮 */
		str_printelem(fp, shield_regist_elem, ELEM_ALL);
		fprintf(fp, "  ");

		/* ԕωւ̑ϐ */
		str_printstate(fp, regist_state, STATE_ALL);
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeflag(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̃tOnf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̃tOnf[^o͂ */
	fprintf(fp, "No.       O               "
				"j  s s "
				"ϐ ϓ ω ϋ ϒn "
				"푰  \n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%03X): %-16s :  %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s   %2s  %-4s  %s\n", i, i,
				mon_getname(i),
				str_getflag(ptr_mondata[i].flag_man),
				str_getflag(ptr_mondata[i].flag_woman),
				str_getflag(ptr_mondata[i].flag_undead),
				str_getflag(ptr_mondata[i].flag_unknown),
				str_getflag(ptr_mondata[i].flag_regist_water),
				str_getflag(ptr_mondata[i].flag_regist_throw),
				str_getflag(ptr_mondata[i].flag_regist_soundwave),
				str_getflag(ptr_mondata[i].flag_regist_gaze),
				str_getflag(ptr_mondata[i].flag_regist_ground),
				str_getrace(ptr_mondata[i].type_race),
				str_gethealing(ptr_mondata[i].type_healing));
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeitem(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "----------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̏ACef[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̏ACef[^o͂ */
	fprintf(fp, "No.       O               "
				"   "
				"100%%              "
				"m            "
				"m            "
				"m\n");
	for (i = 0; i < NUM_MON; i++)
	{
		const char *ptr_name[4];

		for (j = 0; j < 4; j++)
			ptr_name[j] = (ptr_mondata[i].item_no[j] != 0xff) ? item_getname(ptr_mondata[i].item_no[j]) : str_nothing;

		fprintf(fp, "%03d(%03X): %-16s :  %3d,  ", i, i, mon_getname(i), ptr_mondata[i].credit);
		for (j = 0; j < 4; j++)
		{
			if (j != 0)
				fprintf(fp, ", ");
			fprintf(fp, "%-16s", ptr_name[j]);
		}
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeequip(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_nothing = "----------------";

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̑ACef[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̑ACef[^o͂ */
	fprintf(fp, "No.       O               "
				"12 "
				"1             "
				"2             "
				"                "
				"h1             "
				"h2\n");
	for (i = 0; i < NUM_MON; i++)
	{
		const char *ptr_name[5];

		for (j = 0; j < 5; j++)
			ptr_name[j] = (ptr_mondata[i].equipment_no[j] != 0xff) ? item_getname(ptr_mondata[i].equipment_no[j]) : str_nothing;

		fprintf(fp, "%03d(%03X): %-16s : ", i, i, mon_getname(i));
		for (j = 0; j < 2; j++)
			fprintf(fp, " %2s  ", str_getflag(ptr_mondata[i].flag_catch_weapon[j]));
		fprintf(fp, " ");
		for (j = 0; j < 5; j++)
		{
			if (j != 0)
				fprintf(fp, ", ");
			fprintf(fp, "%-16s", ptr_name[j]);
		}
		fputc('\n', fp);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeact(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i, j, k;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̍sp^[f[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̍sp^[f[^o͂ */
	for (i = 0; i < NUM_MON; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%03d(%02X): %s]\n", i, i, mon_getname(i));

		fprintf(fp, "No.   s\n");
		for (j = 0; j < 2; j++)
		{
			for (k = 0; k < 8; k++)
				fprintf(fp, "%1d-%02d: %s\n", j + 1, k + 1, tech_getname(ptr_mondata[i].action_no[j][k] + 0x100));
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeencmain(const char *ptr_filename, const monencmain_t *ptr_monencmain)
{
	int i, j;
	FILE *fp = NULL;
	const char *str_montype[] =
	{
		"[",
		"",
		"b",
		"",
		"bl",
		"A",
		"",
		"",
		"]r",
		"",
		"",
		"@",
		"",
		"",
		"S",
		"d",
	};

	assert(sizeof(str_montype) / sizeof(str_montype[0]) == 16);

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_monencmain == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[oxf[^([_[X^[)o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[oxf[^([_[X^[)o͂ */
	for (i = 0; i < NUM_MONENCMAIN; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%02d(%02X): %s]\n", i, i, str_montype[i]);

		fprintf(fp, "No. X^[          Lv. ő吔 őTCY\n");
		for (j = 0; j < 16; j++)
		{
			fprintf(fp, "%2d: %-16s :  %3d, %5d,    %2d~%2d\n", j,
					mon_getname(ptr_monencmain[i].level[j].monster_no),
					j * 16,
					ptr_monencmain[i].level[j].party.max_num,
					ptr_monencmain[i].level[j].party.max_size.width,
					ptr_monencmain[i].level[j].party.max_size.height);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writeencsub(const char *ptr_filename, const monencsub_t *ptr_monencsub)
{
	int i, j;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_monencsub == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[oxf[^(]X^[)o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[oxf[^(]X^[)o͂ */
	for (i = 0; i < NUM_MONENCSUB; i++)
	{
		if (i != 0)
			fputc('\n', fp);

		fprintf(fp, "[No.%02d(%02X): ^Cv%02d]\n", i, i, i);

		fprintf(fp, "No. X^[          Lv.\n");
		for (j = 0; j < 48; j++)
		{
			fprintf(fp, "%2d: %-16s :  %3d\n", j,
					mon_getname(ptr_monencsub[i].level[j].monster_no),
					ptr_monencsub[i].level[j].monster_level);
		}
	}

	fclose(fp);
	fp = NULL;

	return 1;
}

int mon_writemisc(const char *ptr_filename, const mondata_t *ptr_mondata)
{
	int i;
	FILE *fp = NULL;

	/* `FbN */
	if (ptr_filename == NULL || *ptr_filename == '\0' || ptr_mondata == NULL)
	{
		assert(0);
		return 0;
	}

	/* X^[̃TCỸf[^o͂t@CJ */
	if ((fp = fopen(ptr_filename, "w")) == NULL)
		return 0;

	/* X^[̃TCỸf[^o͂ */
	fprintf(fp, "No.       O               "
				" TCY\n");
	for (i = 0; i < NUM_MON; i++)
	{
		fprintf(fp, "%03d(%03X): %-16s :  %2d~%2d\n", i, i,
				mon_getname(i),
				ptr_mondata[i].size.width,
				ptr_mondata[i].size.height);
	}

	fclose(fp);
	fp = NULL;

	return 1;
}
